package es.csa.commons.faces.util;

/*
 * #%L
 * Librerías comunes de Java Server Faces de CSA
 * %%
 * Copyright (C) 2015 CSA - Centro Regional de Servicios Avanzados S.A.
 * %%
 * This file is part of "Librerías comunes de CSA".
 * 
 * developed by: Centro Regional de Servicios Avanzados, S.A.
 * 
 * "Librerías comunes de CSA" is free software: you can redistribute it and/or modify it under the terms of:
 * - Apache License, Version 2.0 (the "License");  
 * - under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 * 
 * "Librerías comunes de CSA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or General Public License for more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE­2.0
 * 
 * You may obtain a copy of the General Public Licence at:
 * http://www.gnu.org/licenses/
 * #L%
 */
 
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.el.ELContext;
import javax.el.ExpressionFactory;
import javax.el.ValueExpression;
import javax.faces.application.ViewHandler;
import javax.faces.component.UIViewRoot;
import javax.faces.context.FacesContext;
import javax.servlet.http.HttpServletResponse;

import org.richfaces.function.RichFunction;

/**
 * Diferentes utilidades para JSF muchas de ellas para utilizarlas como función EL.
 * 
 * @author Josué Alcalde González
 */
public class FacesUtils {

  /**
   * Evaluar y obtener el valor de una expresión EL.
   * 
   * @param <T>
   *          El tipo que devolverá.
   * @param fctx
   *          El contexto.
   * @param elExpression
   *          Expresión EL a evaluar.
   * @param expectedClass
   *          Clase experada.
   * @return El valor obteniod al evaluar la expresión.
   */
  @SuppressWarnings("unchecked")
  public static <T> T getValueOf(FacesContext fctx, String elExpression, Class<T> expectedClass) {
    if (fctx == null) fctx = FacesContext.getCurrentInstance();
    if (fctx == null) return null;
    ELContext ctx = fctx.getELContext();
    ExpressionFactory expFactory = FacesContext.getCurrentInstance().getApplication().getExpressionFactory();
    ValueExpression valueExp = expFactory.createValueExpression(ctx, elExpression, expectedClass);
    return (T) valueExp.getValue(ctx);
  }

  /**
   * Establecer el valor de una expresión EL.
   * 
   * @param fctx
   *          El contexto.
   * @param elExpression
   *          La expresión EL.
   * @param value
   *          El valor a establecer.
   */
  public static void setValueOf(FacesContext fctx, String elExpression, Object value) {
    if (fctx == null) fctx = FacesContext.getCurrentInstance();
    ELContext ctx = fctx.getELContext();
    ExpressionFactory expFactory = FacesContext.getCurrentInstance().getApplication().getExpressionFactory();
    ValueExpression valueExp = expFactory.createValueExpression(ctx, elExpression, value.getClass());
    valueExp.setValue(ctx, value);
  }

  /**
   * Evaluar y obtener el valor de una expresión EL.
   * 
   * @param <T>
   *          El tipo que devolverá.
   * @param elExpression
   *          Expresión EL a evaluar.
   * @param expectedClass
   *          Clase experada.
   * @return El valor obteniod al evaluar la expresión.
   */
  public static <T> T getValueOf(String elExpression, Class<T> expectedClass) {
    return getValueOf(FacesContext.getCurrentInstance(), elExpression, expectedClass);
  }

  /**
   * Establecer el valor de una expresión EL.
   * 
   * @param elExpression
   *          La expresión EL.
   * @param value
   *          El valor a establecer.
   */
  public static void setValueOf(String elExpression, Object value) {
    setValueOf(FacesContext.getCurrentInstance(), elExpression, value);
  }

  /**
   * Envía una orden de redirección al navegador.
   * 
   * @param fctx
   *          El contexto.
   * @param newAddress
   *          La nueva dirección o vista a la que redirigir.
   */
  public static void redirect(FacesContext fctx, String newAddress) {
    if (fctx == null) fctx = FacesContext.getCurrentInstance();
    // Al ser redirección, cambiar jspx por faces
    HttpServletResponse response = (HttpServletResponse) fctx.getExternalContext().getResponse();
    try {
      String contextPath = fctx.getExternalContext().getRequestContextPath();
      response.sendRedirect(contextPath + newAddress);
    } catch (Exception e) {
      // Que no se renderice esta página bajo ningún concepto, ir a la principal.
      changeViewRoot(fctx, "");
    } finally {
      fctx.responseComplete();
    }
  }

  /**
   * Cambia la vista que se va a renderizar.
   * 
   * @param fctx
   *          El contexto.
   * @param newAddress
   *          La nueva dirección o vista a la que redirigir.
   */
  public static void changeViewRoot(FacesContext facesContext, String newAddress) {
    if (facesContext == null) facesContext = FacesContext.getCurrentInstance();
    if (facesContext == null) return;
    ViewHandler vh = facesContext.getApplication().getViewHandler();
    UIViewRoot newRoot = vh.createView(facesContext, newAddress);
    facesContext.setViewRoot(newRoot);
  }

  /**
   * Envía un error http al cliente.
   * 
   * @param fctx
   *          El contexto.
   * @param error
   *          El número de error.
   */
  public static void sendHttpError(FacesContext fctx, int error) {
    HttpServletResponse response = (HttpServletResponse) fctx.getExternalContext().getResponse();
    try {
      response.sendError(error);
    } catch (Exception e) {
      // Que no se renderice esta página bajo ningún concepto, ir a la página principal
      FacesUtils.changeViewRoot(fctx, "");
    } finally {
      fctx.responseComplete();
    }
  }

  // ------------------------------------------------------------------------------------------------------ EL Functions

  /**
   * Obtener todos los valores de una enumeración como una lista.
   * 
   * @param enumClass
   *          Un tipo enumeración.
   * @return La lista de enumeraciones.
   */
  @SuppressWarnings("unchecked")
  public static List<Enum> getEnumItems(String enumClass) {
    Class<Enum<?>> clazz;
    List<Enum> enums = new ArrayList<Enum>();
    try {
      clazz = (Class<Enum<?>>) Class.forName(enumClass);
      for (Enum e : clazz.getEnumConstants())
        enums.add(e);
      return enums;
    } catch (ClassNotFoundException ex) {
      ex.printStackTrace();
    }
    return null;
  }

  /**
   * Obtener el tamaño de una colección. (Para utilizar desde EL).
   * 
   * @param collection
   *          Colección.
   * @return El tamaño de la colección o 0 si la conexión es nula.
   */
  public static int size(Collection<?> c) {
    return c == null ? 0 : c.size();
  }

  /**
   * Obtener si un objeto es instancia de una clase determinada
   * 
   * @param className
   *          Nombre de la clase
   * @param obj
   *          Objeto que se quiere comprobar
   * @return booleano
   */
  public static boolean instanceOf(Object object, String className) {
    try {

      return Class.forName(className).isInstance(object);
    } catch (ClassNotFoundException e) {
      e.printStackTrace();
      return false;
    }
  }

  public static <T> List<T> toList(Collection<T> collection) {
    List<T> list = new ArrayList<T>();
    list.addAll(collection);
    return list;
  }

  /**
   * Reemplazar las ocurrencias de la a en cadena, por la cadena b
   * 
   * @param regex
   *          string a reemplazar (expresion regular)
   * @param substitution
   *          cadena con la cual se reemplaza
   * @return String con la cadena y los reemplazos
   */
  public static String replaceAll(String string, String regex, String substitution) {
    try {
      string = string.replaceAll(regex, substitution);
    } catch (Exception e) {
      e.printStackTrace();
      return null;
    }
    return string;
  }

  /**
   * Corta una cadena a la longitud especificada
   * 
   * @param cadena
   * @param limit
   * @return
   */
  public static String subString(String string, Integer limit) {
    try {
      if (string.length() < limit) { return string; }
    } catch (Exception e) {}
    return string.substring(0, limit);
  }

  /**
   * Rellena una cadena con el caracter indicado hasta alcanzar el tamaño indicado
   * 
   * @param string
   *          cadena que se rellenara
   * @param character
   *          caracter de relleno
   * @param size
   *          tamaño que debe alcanzar
   * @return string rellenada
   */
  public static String fillString(String string, Character character, Integer size) {
    int lenght = 0;
    if (string != null && string.length() < size) lenght = string.length();
    for (int i = 0; i <= (size - lenght); i++) {
      string = string + character;
    }
    return string;
  }

  /**
   * Concatena dos cadenas.
   * 
   * @param string1
   *          Primera cadena.
   * @param string2
   *          Segunda cadena.
   * @return Las cadenas concatenadas.
   */
  public static String concat(String string1, String string2) {
    return string1.concat(string2);
  }
  
  public static String clientIdJQuery(String id) {
    return RichFunction.clientId(id).replace(":", "\\:");
  }

}
