package es.csa.commons.faces.util;

/*
 * #%L
 * Librerías comunes de Java Server Faces de CSA
 * %%
 * Copyright (C) 2015 CSA - Centro Regional de Servicios Avanzados S.A.
 * %%
 * This file is part of "Librerías comunes de CSA".
 * 
 * developed by: Centro Regional de Servicios Avanzados, S.A.
 * 
 * "Librerías comunes de CSA" is free software: you can redistribute it and/or modify it under the terms of:
 * - Apache License, Version 2.0 (the "License");  
 * - under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 * 
 * "Librerías comunes de CSA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or General Public License for more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE­2.0
 * 
 * You may obtain a copy of the General Public Licence at:
 * http://www.gnu.org/licenses/
 * #L%
 */
 
import static javax.faces.application.FacesMessage.SEVERITY_ERROR;
import static javax.faces.application.FacesMessage.SEVERITY_FATAL;
import static javax.faces.application.FacesMessage.SEVERITY_INFO;
import static javax.faces.application.FacesMessage.SEVERITY_WARN;

import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import javax.faces.FactoryFinder;
import javax.faces.application.Application;
import javax.faces.application.ApplicationFactory;
import javax.faces.application.FacesMessage;
import javax.faces.application.FacesMessage.Severity;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;

/**
 * Funciones para tratamiento de mensajes con jsf, algunas de ellas utilizadas como función EL.
 * 
 * @author Josué Alcalde González
 */
public class MensajesUtils {

  // -------------------------------------------------------------------------------------------------------- Constantes

  /** Clave para el detalle. */
  private static final String DETAIL_KEY = "_detail";

  /** Id del application Factory. */
  public static final String APPLICATION_FACTORY_ID = "javax.faces.application.ApplicationFactory";

  /** Valor por defecto del Message Bundle. */
  public static final String DEFAULT_RESOURCE_BUNDLE = "es.csa.commons.faces.resources.FacesMessages";

  // --------------------------------------------------------------------------------------- Creación de mensajes rápida

  /**
   * Crear un mensaje a partir de un ResourceBundle específico especificando formatos de sumario y detalles.
   * 
   * @param context
   *          El contexto.
   * @param bundle
   *          El resource bundle.
   * @param summaryFormat
   *          Formato del sumario preestablecido.
   * @param detailFormat
   *          Formato del detalle preestablecido.
   * @param severity
   *          La severidad del mensaje.
   * @param messageId
   *          El id del mensaje.
   * @param params
   *          Los parámetros.
   * @return El mensaje.
   */
  public static FacesMessage createMessage(FacesContext context, ResourceBundle bundle, String summaryFormat,
      String detailFormat, FacesMessage.Severity severity, String messageId, String... params) {
    Locale locale = getCurrentLocale(context);
    String summary = summaryFormat;
    String detail = detailFormat;

    if (bundle != null) {
      if (summary == null || summary.length() == 0) {
        try {
          summary = bundle.getString(messageId);
        } catch (MissingResourceException e) {}
      }
      if (detail == null || detail.length() == 0) {
        try {
          detail = bundle.getString(messageId + DETAIL_KEY);
        } catch (MissingResourceException e) {}
      }
    }

    if (summary == null || summary.length() == 0)
      summary = messageId;
    else
      summary = replaceParams(locale, summary, params);

    if (detail == null || detail.length() == 0)
      detail = summary;
    else
      detail = replaceParams(locale, detail, params);

    return new FacesMessage(severity, summary, detail);
  }

  /**
   * Crear un mensaje a partir de un ResourceBundle específico.
   * 
   * @param context
   *          El contexto.
   * @param severity
   *          La severidad del mensaje.
   * @param messageId
   *          El id del mensaje.
   * @param params
   *          Los parámetros.
   * @return El mensaje.
   */
  public static FacesMessage createMessage(FacesContext context, ResourceBundle bundle, FacesMessage.Severity severity,
      String messageId, String... params) {
    return createMessage(context, bundle, null, null, severity, messageId, params);
  }

  /**
   * Crear un mensaje de error fatal a partir de un ResourceBundle específico.
   * 
   * @param context
   *          El contexto.
   * @param messageId
   *          El id del mensaje.
   * @param params
   *          Los parámetros.
   * @return El mensaje.
   */
  public static FacesMessage createFatalMessage(FacesContext context, ResourceBundle bundle, String messageId,
      String... params) {
    return createMessage(context, bundle, FacesMessage.SEVERITY_FATAL, messageId, params);
  }

  /**
   * Crear un mensaje de error a partir de un ResourceBundle específico.
   * 
   * @param context
   *          El contexto.
   * @param bundle
   *          El resource bundle.
   * @param messageId
   *          El id del mensaje.
   * @param params
   *          Los parámetros.
   * @return El mensaje.
   */
  public static FacesMessage createErrorMessage(FacesContext context, ResourceBundle bundle, String messageId,
      String... params) {
    return createMessage(context, bundle, FacesMessage.SEVERITY_ERROR, messageId, params);
  }

  /**
   * Crear un mensaje de aviso a partir de un ResourceBundle específico.
   * 
   * @param context
   *          El contexto.
   * @param messageId
   *          El id del mensaje.
   * @param params
   *          Los parámetros.
   * @return El mensaje.
   */
  public static FacesMessage createWarnMessage(FacesContext context, ResourceBundle bundle, String messageId,
      String... params) {
    return createMessage(context, bundle, FacesMessage.SEVERITY_WARN, messageId, params);
  }

  /**
   * Crear un mensaje de información a partir de un ResourceBundle específico.
   * 
   * @param context
   *          El contexto.
   * @param messageId
   *          El id del mensaje.
   * @param params
   *          Los parámetros.
   * @return El mensaje.
   */
  public static FacesMessage createInfoMessage(FacesContext context, ResourceBundle bundle, String messageId,
      String... params) {
    return createMessage(context, bundle, FacesMessage.SEVERITY_INFO, messageId, params);
  }

  /**
   * Crear un mensaje usando el bundle de faces config y sino el establecido por defecto, utilizando formatos de sumario
   * y detalle específicos si es necesario.
   * 
   * @param context
   *          El contexto.
   * @param summaryFormat
   *          Formato del sumario preestablecido.
   * @param detailFormat
   *          Formato del detalle preestablecido.
   * @param severity
   *          La severidad del mensaje.
   * @param messageId
   *          El id del mensaje.
   * @param params
   *          Los parámetros.
   * @return El mensaje.
   */
  public static FacesMessage createMessage(FacesContext context, String summaryFormat, String detailFormat,
      FacesMessage.Severity severity, String messageId, String... params) {
    Locale locale = getCurrentLocale(context);
    String summary = summaryFormat;
    String detail = detailFormat;
    ResourceBundle bundle = null;

    if (summary == null || summary.length() == 0 || detail == null || detail.length() == 0) {
      bundle = getFacesBundle(context, locale);
      if (bundle != null) {
        if (summary == null || summary.length() == 0) {
          try {
            summary = bundle.getString(messageId);
          } catch (MissingResourceException e) {}
        }
        if (detail == null || detail.length() == 0) {
          try {
            detail = bundle.getString(messageId + DETAIL_KEY);
          } catch (MissingResourceException e) {}
        }
      }
    }

    if (summary == null || summary.length() == 0 || detail == null || detail.length() == 0) {
      bundle = getDefaultBundle(context, locale);
      if (bundle != null) {
        if (summary == null || summary.length() == 0) {
          try {
            summary = bundle.getString(messageId);
          } catch (MissingResourceException e) {}
        }
        if (detail == null || detail.length() == 0) {
          try {
            detail = bundle.getString(messageId + DETAIL_KEY);
          } catch (MissingResourceException e) {}
        }
      }
    }

    if (summary == null || summary.length() == 0)
      summary = messageId;
    else
      summary = replaceParams(locale, summary, params);

    if (detail == null || detail.length() == 0)
      detail = summary;
    else
      detail = replaceParams(locale, detail, params);

    return new FacesMessage(severity, summary, detail);
  }

  /**
   * Crear un mensaje usando el bundle de faces config y sino el establecido por defecto.
   * 
   * @param context
   *          El contexto.
   * @param severity
   *          La severidad del mensaje.
   * @param messageId
   *          El id del mensaje.
   * @param params
   *          Los parámetros.
   * @return El mensaje.
   */
  public static FacesMessage createMessage(FacesContext context, FacesMessage.Severity severity, String messageId,
      String... params) {
    return createMessage(context, null, null, severity, messageId, params);
  }

  /**
   * Crear un mensaje de error fatal usando el bundle de faces config y sino el establecido por defecto.
   * 
   * @param context
   *          El contexto.
   * @param severity
   *          La severidad del mensaje.
   * @param messageId
   *          El id del mensaje.
   * @param params
   *          Los parámetros.
   * @return El mensaje.
   */
  public static FacesMessage createFatalMessage(FacesContext context, FacesMessage.Severity severity, String messageId,
      String... params) {
    return createMessage(context, FacesMessage.SEVERITY_FATAL, messageId, params);
  }

  /**
   * Crear un mensaje de error usando el bundle de faces config y sino el establecido por defecto.
   * 
   * @param context
   *          El contexto.
   * @param messageId
   *          El id del mensaje.
   * @param params
   *          Los parámetros.
   * @return El mensaje.
   */
  public static FacesMessage createErrorMessage(FacesContext context, String messageId, String... params) {
    return createMessage(context, FacesMessage.SEVERITY_ERROR, messageId, params);
  }

  /**
   * Crear un mensaje de aviso usando el bundle de faces config y sino el establecido por defecto.
   * 
   * @param context
   *          El contexto.
   * @param messageId
   *          El id del mensaje.
   * @param params
   *          Los parámetros.
   * @return El mensaje.
   */
  public static FacesMessage createWarnMessage(FacesContext context, String messageId, String... params) {
    return createMessage(context, FacesMessage.SEVERITY_WARN, messageId, params);
  }

  // -------------------------------------------------------------------------------------- Métodos para añadir mensajes

  /**
   * Añade un mensaje al contexto.
   * 
   * @param context
   *          El contexto.
   * @param componentId
   *          El id del componente.
   * @param message
   *          El mensaje.
   */
  public static void addMessage(FacesContext context, String componentId, FacesMessage message) {
    if (context == null) context = FacesContext.getCurrentInstance();
    context.addMessage(componentId, message);
  }

  /**
   * Añade un mensaje al contexto, dado por una cadena y unos parámetros que se sustituyen.
   * 
   * @param context
   *          El contexto.
   * @param severity
   *          La severidad.
   * @param componentId
   *          El id del componente.
   * @param message
   *          El mensaje.
   * @param params
   *          Los parámetros.
   */
  public static void addMessage(FacesContext context, Severity severity, String componentId, String message,
      String... params) {
    if (context == null) context = FacesContext.getCurrentInstance();
    message = replaceParams(getCurrentLocale(context), message, params);
    addMessage(context, componentId, new FacesMessage(severity, message, message));
  }

  /**
   * Añade un mensaje de error fatal al contexto, dado por una cadena y unos parámetros que se sustituyen.
   * 
   * @param context
   *          El contexto.
   * @param componentId
   *          El id del componente.
   * @param message
   *          El mensaje.
   * @param params
   *          Los parámetros.
   */
  public static void addFatalMessage(FacesContext context, String componentId, String message, String... params) {
    addMessage(context, SEVERITY_FATAL, componentId, message);
  }

  /**
   * Añade un mensaje de error al contexto, dado por una cadena y unos parámetros que se sustituyen.
   * 
   * @param context
   *          El contexto.
   * @param componentId
   *          El id del componente.
   * @param message
   *          El mensaje.
   * @param params
   *          Los parámetros.
   */
  public static void addErrorMessage(FacesContext context, String componentId, String message, String... params) {
    addMessage(context, SEVERITY_ERROR, componentId, message);
  }

  /**
   * Añade un mensaje de aviso fatal al contexto, dado por una cadena y unos parámetros que se sustituyen.
   * 
   * @param context
   *          El contexto.
   * @param componentId
   *          El id del componente.
   * @param message
   *          El mensaje.
   * @param params
   *          Los parámetros.
   */
  public static void addInfoMessage(FacesContext context, String componentId, String message, String... params) {
    addMessage(context, SEVERITY_INFO, componentId, message);
  }

  /**
   * Añade un mensaje de información fatal al contexto, dado por una cadena y unos parámetros que se sustituyen.
   * 
   * @param context
   *          El contexto.
   * @param componentId
   *          El id del componente.
   * @param message
   *          El mensaje.
   * @param params
   *          Los parámetros.
   */
  public static void addWarnMessage(FacesContext context, String componentId, String message, String... params) {
    addMessage(context, SEVERITY_WARN, componentId, message);
  }

  // -------------------------------------------------------------------------------------------------- Obtener mensajes

  /**
   * Devuelve una lista de los mensajes de error fatal.
   * 
   * @return Una lista con los mensajes de error fatal.
   */
  public static List<FacesMessage> getFatalMessages() {
    return getMessages(SEVERITY_FATAL);
  }

  /**
   * Devuelve una lista de los mensajes de error.
   * 
   * @return Una lista con los mensajes de error.
   */
  public static List<FacesMessage> getErrorMessages() {
    return getMessages(SEVERITY_ERROR);
  }

  /**
   * Devuelve una lista de los mensajes de aviso.
   * 
   * @return Una lista con los mensajes de aviso.
   */
  public static List<FacesMessage> getWarnMessages() {
    return getMessages(SEVERITY_WARN);
  }

  /**
   * Devuelve una lista de los mensajes de información.
   * 
   * @return Una lista con los mensajes de información.
   */
  public static List<FacesMessage> getInfoMessages() {
    return getMessages(SEVERITY_INFO);
  }

  /**
   * Devuelve una lista de todos los mensajes.
   * 
   * @return Una lista de todos los mensajes.
   */
  public static List<FacesMessage> getMessages() {
    return getMessages(null);
  }

  /**
   * Devuelve una lista de los mensajes de una severidad, o todos si se especifa una severidad nula.
   * 
   * @param severity
   *          La severidad
   * @return Una lista con los mensajes.
   */
  protected static List<FacesMessage> getMessages(FacesMessage.Severity severity) {
    FacesContext context = FacesContext.getCurrentInstance();
    List<FacesMessage> messages = new ArrayList<FacesMessage>();
    Iterator<FacesMessage> fm = context.getMessages();
    while (fm.hasNext()) {
      FacesMessage message = (fm.next());
      if (severity == null || message.getSeverity().equals(severity)) {
        messages.add(message);
      }
    }
    return messages;
  }

  /**
   * Devuelve una lista de los mensajes globales de error fatal.
   * 
   * @return Una lista con los mensajes globales de error fatal.
   */
  public static List<FacesMessage> getFatalGlobalMessages() {
    return getGlobalMessages(SEVERITY_FATAL);
  }

  /**
   * Devuelve una lista de los mensajes globales de error.
   * 
   * @return Una lista con los mensajes globales de error.
   */
  public static List<FacesMessage> getErrorGlobalMessages() {
    return getGlobalMessages(SEVERITY_ERROR);
  }

  /**
   * Devuelve una lista de los mensajes globales de aviso.
   * 
   * @return Una lista con los mensajes globales de aviso.
   */
  public static List<FacesMessage> getWarnGlobalMessages() {
    return getGlobalMessages(SEVERITY_WARN);
  }

  /**
   * Devuelve una lista de los mensajes globales de información.
   * 
   * @return Una lista con los mensajes globales de información.
   */
  public static List<FacesMessage> getInfoGlobalMessages() {
    return getGlobalMessages(SEVERITY_INFO);
  }

  /**
   * Devuelve una lista de todos los mensajes globales.
   * 
   * @return Una lista de todos los mensajes globales.
   */
  public static List<FacesMessage> getGlobalMessages() {
    return getGlobalMessages(null);
  }

  /**
   * Devuelve una lista de los mensajes globales de una severidad, o todos si se especifa una severidad nula.
   * 
   * @param severity
   *          La severidad
   * @return Una lista con los mensajes.
   */
  protected static List<FacesMessage> getGlobalMessages(FacesMessage.Severity severity) {
    FacesContext context = FacesContext.getCurrentInstance();
    List<FacesMessage> messages = new ArrayList<FacesMessage>();
    Iterator<FacesMessage> fm = context.getMessages(null);
    while (fm.hasNext()) {
      FacesMessage message = (fm.next());
      if (severity == null || message.getSeverity().equals(severity)) {
        messages.add(message);
      }
    }
    return messages;
  }

  /**
   * Devuelve si hay mensajes de error fatal.
   * 
   * @return Si hay mensajes de error fatal.
   */
  public static boolean hasFatalMessages() {
    return getFatalMessages().size() > 0;
  }

  /**
   * Devuelve si hay mensajes de error.
   * 
   * @return Si hay mensajes de error.
   */
  public static boolean hasErrorMessages() {
    return getErrorMessages().size() > 0;
  }

  /**
   * Devuelve si hay mensajes de aviso.
   * 
   * @return Si hay mensajes de aviso.
   */
  public static boolean hasWarnMessages() {
    return getWarnMessages().size() > 0;
  }

  /**
   * Devuelve si hay mensajes de información.
   * 
   * @return Si hay mensajes de información.
   */
  public static boolean hasInfoMessages() {
    return getInfoMessages().size() > 0;
  }

  /**
   * Devuelve si hay mensajes.
   * 
   * @return Si hay mensajes.
   */
  public static boolean hasMessages() {
    return getMessages().size() > 0;
  }

  /**
   * Devuelve si hay mensajes globales de error fatal.
   * 
   * @return Si hay mensajes globales de error fatal.
   */
  public static boolean hasFatalGlobalMessages() {
    return getFatalGlobalMessages().size() > 0;
  }

  /**
   * Devuelve si hay mensajes globales de error.
   * 
   * @return Si hay mensajes globales de error.
   */
  public static boolean hasErrorGlobalMessages() {
    return getErrorGlobalMessages().size() > 0;
  }

  /**
   * Devuelve si hay mensajes globales de aviso.
   * 
   * @return Si hay mensajes globales de aviso.
   */
  public static boolean hasWarnGlobalMessages() {
    return getWarnGlobalMessages().size() > 0;
  }

  /**
   * Devuelve si hay mensajes globales de información.
   * 
   * @return Si hay mensajes globales de información.
   */
  public static boolean hasInfoGlobalMessages() {
    return getInfoGlobalMessages().size() > 0;
  }

  /**
   * Devuelve si hay mensajes globales.
   * 
   * @return Si hay mensajes globales.
   */
  public static boolean hasGlobalMessages() {
    return getGlobalMessages().size() > 0;
  }

  /**
   * Devuelve si hay mensajes globales de error fatal.
   * 
   * @return Si hay mensajes globales de error fatal.
   */
  public static boolean hasFatalComponentMessages() {
    return !hasFatalGlobalMessages() && hasFatalMessages();
  }

  /**
   * Devuelve si hay mensajes globales de error.
   * 
   * @return Si hay mensajes globales de error.
   */
  public static boolean hasErrorComponentMessages() {
    return !hasErrorGlobalMessages() && hasErrorMessages();
  }

  /**
   * Devuelve si hay mensajes globales de aviso.
   * 
   * @return Si hay mensajes globales de aviso.
   */
  public static boolean hasWarnComponentMessages() {
    return !hasWarnGlobalMessages() && hasWarnMessages();
  }

  /**
   * Devuelve si hay mensajes globales de información.
   * 
   * @return Si hay mensajes globales de información.
   */
  public static boolean hasInfoComponentMessages() {
    return !hasInfoGlobalMessages() && hasInfoMessages();
  }

  /**
   * Devuelve si hay mensajes globales.
   * 
   * @return Si hay mensajes globales.
   */
  public static boolean hasComponentMessages() {
    return !hasGlobalMessages() && hasMessages();
  }

  /**
   * Funcion que devuelve si un elemento tiene errores del tipo info
   * 
   * @return
   */
  public static boolean hasInfoMessages(String clientId) {
    return hasMessages(clientId, FacesMessage.SEVERITY_INFO);
  }

  /**
   * Funcion que devuelve si un elemento tiene errores del tipo warning
   * 
   * @return
   */
  public static boolean hasWarnMessages(String clientId) {
    return hasMessages(clientId, FacesMessage.SEVERITY_WARN);
  }

  /**
   * Funcion que devuelve si un elemento tiene errores del tipo error
   * 
   * @return
   */
  public static boolean hasErrorMessages(String clientId) {
    return hasMessages(clientId, FacesMessage.SEVERITY_ERROR);
  }

  /**
   * Funcion que devuelve si un elemento tiene errores del tipo fatal
   * 
   * @return
   */
  public static boolean hasFatalMessages(String clientId) {
    return hasMessages(clientId, FacesMessage.SEVERITY_FATAL);
  }

  /**
   * Funcion que devuelve si un elemento tiene errores del tipo warning
   * 
   * @return
   */
  public static boolean hasMessages(String clientId) {
    return hasMessages(clientId);
  }

  /**
   * Funcion que devuelve si un elemento tiene errores del tipo warning
   * 
   * @return
   */
  public static boolean hasMessages(String clientId, Severity severity) {
    FacesContext context = FacesContext.getCurrentInstance();
    Iterator<FacesMessage> fm = context.getMessages(clientId);
    while (fm.hasNext()) {
      FacesMessage message = (fm.next());
      if (severity == null)
        return true;
      else if (message.getSeverity().equals(severity)) return true;
    }
    return false;
  }

  // ----------------------------------------------------------------------------------------------- Métodos de Utilidad

  /**
   * Crear un message de información usando el bundle de faces config y sino el establecido por defecto.
   * 
   * @param context
   *          El contexto.
   * @param messageId
   *          El id del message.
   * @param params
   *          Los parámetros.
   * @return El message.
   */
  public static FacesMessage createInfoMessage(FacesContext context, String messageId, String... params) {
    return createMessage(context, FacesMessage.SEVERITY_INFO, messageId, params);
  }

  /**
   * Obtener message de un resource bundle sustituyendo parámetros.
   * <p>
   * Se busca por orden en el bundle especificado por parámetro, en el bundle especificado por la aplicación faces y en
   * el bundle por defecto.
   * </p>
   * 
   * @param facesContext
   *          El contexto.
   * @param messageId
   *          Id del message.
   * @param bundleName
   *          Nombre del resource bundle (opcional).
   * @return El message que corresponda con los parámetros sustituidos.
   */
  public static String getString(FacesContext facesContext, String messageId, String bundleName) {
    ResourceBundle bundle = null;
    String message = null;

    Locale currentLocale = getCurrentLocale(facesContext);

    if (bundleName != null) {
      bundle = getNamedBundle(facesContext, currentLocale, bundleName);
      try {
        if (bundle != null) message = bundle.getString(messageId);
      } catch (MissingResourceException e) {}
    }
    if (message == null) {
      bundle = getFacesBundle(facesContext, currentLocale);
      try {
        if (bundle != null) message = bundle.getString(messageId);
      } catch (MissingResourceException e) {}
    }
    if (message == null) {
      bundle = getDefaultBundle(facesContext, currentLocale);
      try {
        if (bundle != null) message = bundle.getString(messageId);
      } catch (MissingResourceException e) {}
    }

    return message;
  }

  /**
   * Devuelve el <code>ResourceBundle</code>.
   * 
   * @param facesContext
   *          El contexto.
   * @param locale
   *          La <code>locale</code>.
   * @return El <code>ResourceBundle</code>.
   */
  public static ResourceBundle getFacesBundle(FacesContext facesContext, Locale locale) {
    return ResourceBundle.getBundle(getApplication(facesContext).getMessageBundle(), locale);
  }

  /**
   * Devuelve el <code>ResourceBundle</code> por defecto.
   * 
   * @param facesContext
   *          El contexto.
   * @param locale
   *          La <code>locale</code>.
   * @return El <code>ResourceBundle</code> por defecto.
   */
  public static ResourceBundle getDefaultBundle(FacesContext facesContext, Locale locale) {
    return getNamedBundle(facesContext, locale, DEFAULT_RESOURCE_BUNDLE);
  }

  /**
   * Devuelve el <code>ResourceBundle</code> para un nombre de recurso.
   * 
   * @param facesContext
   *          El contexto.
   * @param locale
   *          La <code>locale</code>.
   * @param resourceBundle
   *          El nombre de recurso.
   * @return El <code>ResourceBundle</code> para un nombre de recurso.
   */
  public static ResourceBundle getNamedBundle(FacesContext facesContext, Locale locale, String bundleName) {
    return ResourceBundle.getBundle(bundleName, locale);
  }

  /**
   * Reemplazar los parámetros de una cadena usando <code>MessageFormat</code>.
   * 
   * @param locale
   *          La <code>Locale</code> a usar en la sustitución.
   * @param message
   *          El message parametrizado original.
   * @param params
   *          Los parámetros que se sustituirán en el mensaje.
   * @return La cadena con los parámetros sustituidos.
   */
  public static String replaceParams(Locale locale, String message, Object params[]) {
    if (params == null || message == null || locale == null) return message;
    MessageFormat mf = new MessageFormat(message);
    return mf.format(((params)));
  }

  /**
   * Devuelve la <code>Application</code>.
   * 
   * @param facesContext
   *          El contexto.
   * @return La <code>Application</code>.
   */
  public static Application getApplication(FacesContext facesContext) {
    Application application = null;
    ApplicationFactory applicationFactory = null;
    if (facesContext == null) facesContext = FacesContext.getCurrentInstance();
    if (facesContext != null) application = facesContext.getApplication();
    if (application == null) {
      applicationFactory = ((ApplicationFactory) FactoryFinder.getFactory(APPLICATION_FACTORY_ID));
      application = applicationFactory.getApplication();
    }
    return application;
  }

  /**
   * Devolver la <code>Locale</code> para la actual petición.
   * <p>
   * Primero, probar con la locale en el <code>ViewRoot</code> del <code>FacesContext</code> y si no utilizar la
   * <code>Locale</code> de la máquina virtual.
   * </p>
   * 
   * @param facesContext
   *          El <code>FacesContext</code> actual.
   * @return La <code>Locale</code> actual.
   */
  public static Locale getCurrentLocale(FacesContext facesContext) {
    Locale locale = null;
    if (facesContext == null) facesContext = FacesContext.getCurrentInstance();
    if (facesContext != null && facesContext.getViewRoot() != null) locale = facesContext.getViewRoot().getLocale();
    if (locale == null) locale = Locale.getDefault();
    return locale;
  }

  /**
   * Obtener la etiqueta del componente y si no tiene, devolver la etiqueta.
   * <p>
   * Basada en un método de jsf-ri.
   * </p>
   * 
   * @param context
   *          El <code>FacesContext</code>.
   * @param component
   *          El component.
   * @return La etiqueta si existe, y si no, el id del componente.
   */
  public static String getLabel(FacesContext context, UIComponent component) {
    Object label = component.getAttributes().get("label");
    if (label == null || (label instanceof String && ((String) label).length() == 0))
      label = component.getValueExpression("label");
    if (label == null || (label instanceof String && ((String) label).length() == 0)) label = component.getId();
    return label == null ? null : label.toString();
  }

}
