package es.csa.commons.persistence.browser.hibernatecriteria;

/*
 * #%L
 * Librerías comunes de Java Server Faces de CSA
 * %%
 * Copyright (C) 2015 CSA - Centro Regional de Servicios Avanzados S.A.
 * %%
 * This file is part of "Librerías comunes de CSA".
 * 
 * developed by: Centro Regional de Servicios Avanzados, S.A.
 * 
 * "Librerías comunes de CSA" is free software: you can redistribute it and/or modify it under the terms of:
 * - Apache License, Version 2.0 (the "License");  
 * - under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 * 
 * "Librerías comunes de CSA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or General Public License for more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE­2.0
 * 
 * You may obtain a copy of the General Public Licence at:
 * http://www.gnu.org/licenses/
 * #L%
 */
 
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.text.Collator;
import java.text.DateFormat;
import java.text.NumberFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.StringTokenizer;

import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.Disjunction;
import org.hibernate.criterion.Junction;
import org.hibernate.criterion.MatchMode;
import org.hibernate.criterion.Restrictions;

import es.csa.commons.persistence.browser.Filter;
import es.csa.commons.util.ClassUtils;

public class FilterHibernateImpl<E> implements Filter<Criterion, E> {

  private static final long serialVersionUID = -7981723317586071447L;

  /** Enumeración de operadores booleanos. */
  public static enum OpBoolean {
    EQUAL,
    EMPTY;
  }

  /** Enumeración de operadores para cadenas. */
  public static enum OpString {
    CONTAINS,
    EQUAL,
    EMPTY;
  }

  /** Enumeración de operadores caracteres. */
  public static enum OpCharacter {
    EQUAL,
    EMPTY;
  }

  /** Enumeración de operadores para enumeraciones. */
  public static enum OpEnum {
    CONTAINS,
    EQUAL,
    EMPTY;
  }

  /** Enumeración de operadores para fechas. */
  public static enum OpDate {
    GREATER,
    LOWER,
    EQUAL,
    GREATER_OR_EQUAL,
    LOWER_OR_EQUAL,
    EMPTY,
    BETWEEN,
    RANGE;
  }

  /** Enumeración de operadores para números. */
  public static enum OpNumber {
    GREATER,
    LOWER,
    EQUAL,
    GREATER_OR_EQUAL,
    LOWER_OR_EQUAL,
    EMPTY,
    BETWEEN,
    RANGE;
  }

  // Provided data

  /** Propiedad especificada al hacer el filtro. */
  protected String filterProperty;

  /** Valor de la cadena del filtro. */
  protected String filterString;

  // Internal data

  /** La operación del filtro una vez parseada la cadena del filtro. */
  @SuppressWarnings("unchecked")
  protected Enum filterOp;

  /** La operación del filtro una vez parseada la cadena del filtro. */
  protected Object filterData;

  /** La propiedad del filtro recortada solo a dos (para poder usar con hibernate). */
  protected String filterCutProperty;

  /** La clase de la propiedad del filtro. */
  protected Class<?> filterPropertyClass;

  /** La clase del tipo de entidad sobre el que se aplica el filtro. */
  protected Class<E> entityClass;

  /** El criterion de hibernate. */
  protected transient Criterion filterCriterion;

  {
    
  }
  
  public FilterHibernateImpl() {
    super();
    ParameterizedType genericSuperclass = (ParameterizedType) getClass().getGenericSuperclass();
    Type[] actualTypeArguments = genericSuperclass.getActualTypeArguments();
    @SuppressWarnings("unchecked")
    Class<E> entityClass = (Class<E>) actualTypeArguments[0];
    this.entityClass = entityClass;
    filterCriterion = null;
  }
  
  public FilterHibernateImpl(Class<E> entityClass) {
    super();
    this.entityClass = entityClass;
    filterCriterion = null;
  }

  public Class<E> getEntityClass() {
    return entityClass;
  }

  public void setEntityClass(Class<E> entityClass) {
    this.entityClass = entityClass;
  }

  public String getFilterProperty() {
    return filterProperty;
  }

  public void setFilterProperty(String filterProperty) {
    this.filterCriterion = null; // lo invalida
    this.filterProperty = filterProperty;
    this.filterCutProperty = ClassUtils.cutProperty(filterProperty);
    this.filterPropertyClass = ClassUtils.getPropertyClass(filterProperty, entityClass);
  }

  public String getFilterString() {
    return filterString;
  }

  public void setFilterString(String filterString) {
    this.filterCriterion = null; // lo invalida
    this.filterString = filterString;
  }
  
  @Override
  public Criterion getCriterion() {
    if (filterCriterion == null) {
      parseFilter();
      if (ClassUtils.isSubclass(filterPropertyClass, Number.class)) {
        filterCriterion = getNumberCriterion();
      } else if (ClassUtils.isSubclass(filterPropertyClass, Boolean.class)) {
        filterCriterion = getBooleanCriterion();
      } else if (ClassUtils.isSubclass(filterPropertyClass, String.class)) {
        filterCriterion = getStringCriterion();
      } else if (ClassUtils.isSubclass(filterPropertyClass, Date.class)) {
        filterCriterion = getDateCriterion();
      } else if (ClassUtils.isSubclass(filterPropertyClass, Character.class)) {
        filterCriterion = getCharacterCriterion();
      } else if (ClassUtils.isSubclass(filterPropertyClass, Enum.class)) {
        filterCriterion = getEnumCriterion();
      } else return null;
    }
    return filterCriterion;
  }

  void parseFilter() {
    if (ClassUtils.isSubclass(filterPropertyClass, Number.class)) {
      parseNumber();
    } else if (ClassUtils.isSubclass(filterPropertyClass, Boolean.class)) {
      parseBoolean();
    } else if (ClassUtils.isSubclass(filterPropertyClass, String.class)) {
      parseString();
    } else if (ClassUtils.isSubclass(filterPropertyClass, Date.class)) {
      parseDate();
    } else if (ClassUtils.isSubclass(filterPropertyClass, Character.class)) {
      parseCharacter();
    } else if (ClassUtils.isSubclass(filterPropertyClass, Enum.class)) {
      parseEnum();
    } else return;
  }

  // ------------------------------------------------------------------------------------------------- Filtros de fechas

  /** Parsea un filtro para una propiedad de tipo fecha. */
  private void parseDate() {
    // Parsear filterOp
    String filterString = this.filterString.trim();
    if (filterString == null || filterString.isEmpty() || filterString.charAt(0) == '!') {
      filterOp = OpDate.EMPTY;
      return;
    }
    // Parsear rangos
    if (filterString.startsWith("[") && filterString.endsWith("]")) {
      filterOp = OpDate.RANGE;
      this.filterData = auxParseDateRange(filterString.substring(1, filterString.length() - 1).trim());
      return;
    }
    // Al menos dos caracters, filterOp y filterData, sino, saltarse esto
    if (filterString.length() >= 2) {
      char primero = filterString.charAt(0);
      char segundo = filterString.charAt(1);
      switch (primero) {
        case '<':
          filterOp = segundo == '=' ? OpDate.LOWER_OR_EQUAL : OpDate.LOWER;
          filterString = segundo == '=' ? filterString.substring(2) : filterString.substring(1);
          break;
        case '>':
          filterOp = segundo == '=' ? OpDate.GREATER_OR_EQUAL : OpDate.GREATER;
          filterString = segundo == '=' ? filterString.substring(2) : filterString.substring(1);
          break;
        case '=':
          filterString = filterString.substring(1);
        default:
          filterOp = OpDate.EQUAL;
          break;
      }
      // Si no llega a dos caracteres (osea, 1 o ninguno).
    } else {
      filterOp = OpDate.EQUAL;
    }
    filterString = filterString.trim();
    try {
      Date fecha = auxParseSimpleDate(filterString);
      filterData = fecha;
    } catch (ParseException e) {
      filterData = null;
    }
  }

  private Object auxParseDateRange(String strDateRange) {
    List<Date[]> ranges = new ArrayList<Date[]>();
    StringTokenizer st = new StringTokenizer(strDateRange, ",;");
    while (st.hasMoreElements()) {
      StringTokenizer st2 = new StringTokenizer(st.nextToken().trim(), "-");
      if (st2.countTokens() > 2) continue;
      try {
        if (st2.countTokens() == 1) {
          Date[] rango = { auxParseSimpleDate(st2.nextToken()), null };
          ranges.add(rango);
        }
        if (st2.countTokens() == 2) {
          Date[] rango = { auxParseSimpleDate(st2.nextToken()), auxParseSimpleDate(st2.nextToken()) };
          ranges.add(rango);
        }
      } catch (ParseException e) {
        // No hacer nada
      }
    }
    return ranges;
  }

  /** Parsea fechas en diferentes formatos. */
  private Date auxParseSimpleDate(String strDate) throws ParseException {
    ParseException ex = null;
    // Probar diferentes opciones
    Date fecha = null;
    // Formato por defecto de la locale (medio)
    DateFormat df = DateFormat.getDateInstance();
    try {
      fecha = df.parse(strDate);
    } catch (ParseException e) {
      ex = e;
    }
    if (fecha != null) return fecha;
    // Si no ha funcionado, utilizar el corto (short)
    df = DateFormat.getDateInstance(DateFormat.SHORT);
    try {
      fecha = df.parse(strDate);
    } catch (ParseException e) {}
    if (fecha != null) return fecha;
    // Probar con nuestro formato tradicional dd/MM/yyyy
    df = new SimpleDateFormat("dd/MM/yyyy");
    try {
      fecha = df.parse(strDate);
    } catch (ParseException e) {}
    if (fecha != null) return fecha;
    // Otro formato
    df = new SimpleDateFormat("dd-MM-yy");
    try {
      fecha = df.parse(strDate);
    } catch (ParseException e) {}
    if (fecha != null) return fecha;
    // Y por dar más opciones, otro formato
    df = new SimpleDateFormat("dd-MM-yyyy");
    try {
      fecha = df.parse(strDate);
    } catch (ParseException e) {}
    if (fecha != null) return fecha;
    // Si no se ha conseguido lanzar la primera excepción.
    throw ex;
  }

  /** Obtener un criterio de un filtro de fechas. */
  @SuppressWarnings("unchecked")
  Criterion getDateCriterion() {
    OpDate operadorFechas = (OpDate) filterOp;
    if (operadorFechas == OpDate.EMPTY) return Restrictions.isNull(filterCutProperty);
    if (filterData == null) return null;
    switch (operadorFechas) {
      case EQUAL:
        return Restrictions.eq(filterCutProperty, (filterData));
      case LOWER:
        return Restrictions.lt(filterCutProperty, (filterData));
      case GREATER:
        return Restrictions.gt(filterCutProperty, (filterData));
      case LOWER_OR_EQUAL:
        return Restrictions.le(filterCutProperty, (filterData));
      case GREATER_OR_EQUAL:
        return Restrictions.ge(filterCutProperty, (filterData));
      case RANGE:
      case BETWEEN:
        Junction disj = Restrictions.disjunction();
        for (Date[] darray : (List<Date[]>) filterData) {
          Date lo = darray[0];
          Date hi = darray[1];
          if (darray[1] == null) {
            disj.add(Restrictions.eq(filterCutProperty, lo));
          } else {
            disj.add(Restrictions.between(filterCutProperty, lo, hi));
          }
        }
        return disj;
      default:
        return null;
    }

  }

  // ------------------------------------------------------------------------------------------------ Filtros de números

  /** Parsea un filtro para una propiedad de tipo numérico. */
  private void parseNumber() {
    // Parsear filterOp
    String filterString = this.filterString.trim();
    if (filterString == null || filterString.isEmpty() || filterString.charAt(0) == '!') {
      filterOp = OpNumber.EMPTY;
      return;
    }
    // Parsear rangos (solo para enteros: short, integer y long)
    if (filterString.startsWith("[") && filterString.endsWith("]")) {
      filterOp = OpNumber.RANGE;
      this.filterData = auxParseNumberRange(filterString.substring(1, filterString.length() - 1).trim());
      return;
    }
    // Al menos dos caracters, filterOp y filterData, sino, saltarse esto
    if (filterString.length() >= 2) {
      char primero = filterString.charAt(0);
      char segundo = filterString.charAt(1);
      switch (primero) {
        case '<':
          filterOp = segundo == '=' ? OpNumber.LOWER_OR_EQUAL : OpNumber.LOWER;
          filterString = segundo == '=' ? filterString.substring(2) : filterString.substring(1);
          break;
        case '>':
          filterOp = segundo == '=' ? OpNumber.GREATER_OR_EQUAL : OpNumber.GREATER;
          filterString = segundo == '=' ? filterString.substring(2) : filterString.substring(1);
          break;
        case '=':
          filterString = filterString.substring(1);
        default:
          filterOp = OpNumber.EQUAL;
          break;
      }
      // Si no llega a dos caracteres (osea, 1 o ninguno).
    } else {
      filterOp = OpNumber.EQUAL;
    }
    filterString = filterString.trim();
    try {
      Number numero = NumberFormat.getNumberInstance().parse(filterString);
      filterData = numero;
    } catch (ParseException e) {
      filterData = null;
    }
  }

  private Object auxParseNumberRange(String numberRange) {
    List<Number[]> ranges = new ArrayList<Number[]>();
    if (filterPropertyClass == Integer.class || filterPropertyClass == Long.class || filterPropertyClass == Short.class
        || filterPropertyClass == Byte.class) {
      StringTokenizer st = new StringTokenizer(numberRange, ",;");
      while (st.hasMoreElements()) {
        StringTokenizer st2 = new StringTokenizer(st.nextToken().trim(), "-");
        if (st2.countTokens() > 2) continue;
        try {
          if (st2.countTokens() == 1) {
            Long[] range = { Long.parseLong(st2.nextToken()), null };
            ranges.add(range);
          }
          if (st2.countTokens() == 2) {
            Long[] range = { Long.parseLong(st2.nextToken()), Long.parseLong(st2.nextToken()) };
            ranges.add(range);
          }
        } catch (NumberFormatException e) {
          // No hacer nada
        }
      }
      return ranges;
    }
    if (filterPropertyClass == Float.class || filterPropertyClass == Double.class) {
      StringTokenizer st = new StringTokenizer(numberRange, ";");
      while (st.hasMoreElements()) {
        StringTokenizer st2 = new StringTokenizer(st.nextToken().trim(), "-");
        if (st2.countTokens() > 2) continue;
        try {
          if (st2.countTokens() == 1) {
            Double[] rango = { Double.parseDouble(st2.nextToken()), null };
            ranges.add(rango);
          }
          if (st2.countTokens() == 2) {
            Double[] rango = { Double.parseDouble(st2.nextToken()), Double.parseDouble(st2.nextToken()) };
            ranges.add(rango);
          }
        } catch (NumberFormatException e) {
          // No hacer nada
        }
      }
      return ranges;
    }
    return ranges;
  }

  /** Aplicar un filterData de números. */
  @SuppressWarnings("unchecked")
  Criterion getNumberCriterion() {
    OpNumber operadorNumeros = (OpNumber) filterOp;
    if (operadorNumeros == OpNumber.EMPTY) return Restrictions.isNull(filterCutProperty);
    if (filterData == null) return null;
    if (filterPropertyClass == Byte.class) {
      switch (operadorNumeros) {
        case EQUAL:
          return Restrictions.eq(filterCutProperty, ((Number) filterData).byteValue());
        case LOWER:
          return Restrictions.lt(filterCutProperty, ((Number) filterData).byteValue());
        case GREATER:
          return Restrictions.gt(filterCutProperty, ((Number) filterData).byteValue());
        case LOWER_OR_EQUAL:
          return Restrictions.le(filterCutProperty, ((Number) filterData).byteValue());
        case GREATER_OR_EQUAL:
          return Restrictions.ge(filterCutProperty, ((Number) filterData).byteValue());
      }
    } else if (filterPropertyClass == Short.class) {
      switch (operadorNumeros) {
        case EQUAL:
          return Restrictions.eq(filterCutProperty, ((Number) filterData).shortValue());
        case LOWER:
          return Restrictions.lt(filterCutProperty, ((Number) filterData).shortValue());
        case GREATER:
          return Restrictions.gt(filterCutProperty, ((Number) filterData).shortValue());
        case LOWER_OR_EQUAL:
          return Restrictions.le(filterCutProperty, ((Number) filterData).shortValue());
        case GREATER_OR_EQUAL:
          return Restrictions.ge(filterCutProperty, ((Number) filterData).shortValue());
        case RANGE:
        case BETWEEN:
          Junction disj = Restrictions.disjunction();
          for (Number[] narray : (List<Number[]>) filterData) {
            Short lo = narray[0].shortValue();
            Short hi = narray[1] == null ? null : narray[1].shortValue();
            if (narray[1] == null) {
              disj.add(Restrictions.eq(filterCutProperty, lo));
            } else {
              disj.add(Restrictions.between(filterCutProperty, lo, hi));
            }
          }
          return disj;
      }
    } else if (filterPropertyClass == Integer.class) {
      switch (operadorNumeros) {
        case EQUAL:
          return Restrictions.eq(filterCutProperty, ((Number) filterData).intValue());
        case LOWER:
          return Restrictions.lt(filterCutProperty, ((Number) filterData).intValue());
        case GREATER:
          return Restrictions.gt(filterCutProperty, ((Number) filterData).intValue());
        case LOWER_OR_EQUAL:
          return Restrictions.le(filterCutProperty, ((Number) filterData).intValue());
        case GREATER_OR_EQUAL:
          return Restrictions.ge(filterCutProperty, ((Number) filterData).intValue());
        case RANGE:
        case BETWEEN:
          Junction disj = Restrictions.disjunction();
          for (Number[] narray : (List<Number[]>) filterData) {
            Integer lo = narray[0].intValue();
            Integer hi = narray[1] == null ? null : narray[1].intValue();
            if (narray[1] == null) {
              disj.add(Restrictions.eq(filterCutProperty, lo));
            } else {
              disj.add(Restrictions.between(filterCutProperty, lo, hi));
            }
          }
          return disj;
      }
    } else if (filterPropertyClass == Long.class) {
      switch (operadorNumeros) {
        case EQUAL:
          return Restrictions.eq(filterCutProperty, ((Number) filterData).longValue());
        case LOWER:
          return Restrictions.lt(filterCutProperty, ((Number) filterData).longValue());
        case GREATER:
          return Restrictions.gt(filterCutProperty, ((Number) filterData).longValue());
        case LOWER_OR_EQUAL:
          return Restrictions.le(filterCutProperty, ((Number) filterData).longValue());
        case GREATER_OR_EQUAL:
          return Restrictions.ge(filterCutProperty, ((Number) filterData).longValue());
        case RANGE:
        case BETWEEN:
          Junction disj = Restrictions.disjunction();
          for (Number[] narray : (List<Number[]>) filterData) {
            Long lo = narray[0].longValue();
            Long hi = narray[1] == null ? null : narray[1].longValue();
            if (narray[1] == null) {
              disj.add(Restrictions.eq(filterCutProperty, lo));
            } else {
              disj.add(Restrictions.between(filterCutProperty, lo, hi));
            }
          }
          return disj;
      }
    } else if (filterPropertyClass == Float.class) {
      switch (operadorNumeros) {
        case EQUAL:
          return Restrictions.eq(filterCutProperty, ((Number) filterData).floatValue());
        case LOWER:
          return Restrictions.lt(filterCutProperty, ((Number) filterData).floatValue());
        case GREATER:
          return Restrictions.gt(filterCutProperty, ((Number) filterData).floatValue());
        case LOWER_OR_EQUAL:
          return Restrictions.le(filterCutProperty, ((Number) filterData).floatValue());
        case GREATER_OR_EQUAL:
          return Restrictions.ge(filterCutProperty, ((Number) filterData).floatValue());
        case RANGE:
        case BETWEEN:
          Junction disj = Restrictions.disjunction();
          for (Number[] narray : (List<Number[]>) filterData) {
            Float lo = narray[0].floatValue();
            Float hi = narray[1] == null ? null : narray[1].floatValue();
            if (narray[1] == null) {
              disj.add(Restrictions.eq(filterCutProperty, lo));
            } else {
              disj.add(Restrictions.between(filterCutProperty, lo, hi));
            }
          }
          return disj;
      }
    } else if (filterPropertyClass == Double.class) {
      switch (operadorNumeros) {
        case EQUAL:
          return Restrictions.eq(filterCutProperty, ((Number) filterData).doubleValue());
        case LOWER:
          return Restrictions.lt(filterCutProperty, ((Number) filterData).doubleValue());
        case GREATER:
          return Restrictions.gt(filterCutProperty, ((Number) filterData).doubleValue());
        case LOWER_OR_EQUAL:
          return Restrictions.le(filterCutProperty, ((Number) filterData).doubleValue());
        case GREATER_OR_EQUAL:
          return Restrictions.ge(filterCutProperty, ((Number) filterData).doubleValue());
        case RANGE:
        case BETWEEN:
          Junction disj = Restrictions.disjunction();
          for (Number[] narray : (List<Number[]>) filterData) {
            Double lo = narray[0].doubleValue();
            Double hi = narray[1] == null ? null : narray[1].doubleValue();
            if (narray[1] == null) {
              disj.add(Restrictions.eq(filterCutProperty, lo));
            } else {
              disj.add(Restrictions.between(filterCutProperty, lo, hi));
            }
          }
          return disj;
      }
    }
    return null;
  }

  // ------------------------------------------------------------------------------------------------ Filtros de cadenas

  /** Parsea un filtro para una propiedad de tipo cadena. */
  private void parseString() {
    // Parsear filterOp
    String filterString = this.filterString.trim();
    if (filterString == null || filterString.isEmpty() || filterString.startsWith("!")) {
      this.filterOp = OpString.EMPTY;
      return;
    }

    List<String> strings = new ArrayList<String>();

    if (filterString.startsWith("=")) {
      this.filterOp = OpString.EQUAL;
      filterString = filterString.substring(1).trim();
      strings.add(filterString);
    } else if (filterString.startsWith("==")) {
      this.filterOp = OpString.EQUAL;
      filterString = filterString.substring(2).trim();
      strings.add(filterString);

    } else {
      this.filterOp = OpString.CONTAINS;
      StringTokenizer st = new StringTokenizer(filterString);
      while (st.hasMoreTokens()) {
        strings.add(st.nextToken());
      }
    }
    this.filterData = strings;
  }

  /** Obtener un criterio a partir de un filtro de cadenas. */
  @SuppressWarnings("unchecked")
  Criterion getStringCriterion() {
    OpString operadorCadenas = (OpString) filterOp;
    List<String> cadenas = (List<String>) filterData;
    switch (operadorCadenas) {
      case EMPTY:
        Disjunction disjunction = Restrictions.disjunction();
        disjunction.add(Restrictions.isNull(filterCutProperty)).add(Restrictions.eq(filterCutProperty, ""));
        return disjunction;
      case EQUAL:
        return Restrictions.ilike(filterCutProperty, cadenas.get(0), MatchMode.EXACT);
      case CONTAINS:
        Junction disj = Restrictions.conjunction();
        for (String cad : cadenas) {
          disj = disj.add(Restrictions.ilike(filterCutProperty, cad, MatchMode.ANYWHERE));
        }
        return disj;
      default:
        return null;
    }
  }

  // ------------------------------------------------------------------------------------------ Filtros de enumeraciones

  /** Parsea un filtro para una propiedad de tipo enum. */
  private void parseEnum() {
    // Parsear filterOp
    String filterString = this.filterString.trim();
    if (filterString == null || filterString.isEmpty() || filterString.startsWith("!")) {
      this.filterOp = OpEnum.EMPTY;
      return;
    }

    List<Enum<?>> enums = new ArrayList<Enum<?>>();
    Collator collator = Collator.getInstance();
    collator.setStrength(Collator.PRIMARY);

    if (filterString.startsWith("=")) {
      this.filterOp = OpEnum.EQUAL;
      filterString = filterString.substring(1).trim();
      for (Object e : filterPropertyClass.getEnumConstants()) {
        if (collator.compare(e.toString(), filterString) == 0) enums.add((Enum<?>) e);
        else if (filterString.equals(((Enum<?>) e).name())) enums.add((Enum<?>) e);
      }
    } else {
      this.filterOp = OpEnum.CONTAINS;
      StringTokenizer st = new StringTokenizer(filterString);
      while (st.hasMoreTokens()) {
        String next = st.nextToken();
        for (Object e : filterPropertyClass.getEnumConstants())
          if (e.toString().toLowerCase().contains(next.toLowerCase())
              || ((Enum<?>) e).name().toLowerCase().contains(next.toLowerCase())) enums.add((Enum<?>) e);
      }
    }
    this.filterData = enums;
  }

  /** Obtener un criterio a partir de un filtro de enumeraciones. */
  @SuppressWarnings("unchecked")
  Criterion getEnumCriterion() {
    OpEnum operadorEnums = (OpEnum) filterOp;
    switch (operadorEnums) {
      case EMPTY:
        return Restrictions.isNull(filterCutProperty);
      case EQUAL:
      case CONTAINS:
        List<Enum> enums = (List<Enum>) filterData;
        return enums == null || enums.size() <= 0 ? null : Restrictions.in(filterCutProperty, enums);
      default:
        return null;
    }
  }

  // --------------------------------------------------------------------------------------------- Filtros de caracteres

  /** Parsea un filtro para una propiedad de tipo char o Character. */
  private void parseCharacter() {
    // Parsear filterOp
    String filterString = this.filterString.trim();
    if (filterString == null || filterString.isEmpty() || filterString.startsWith("!")) {
      this.filterOp = OpCharacter.EMPTY;
      return;
    }
    if (filterString.startsWith("=")) filterString = filterString.substring(1).trim();

    // Tomamos simplemente el primer caracter si existe.
    this.filterData = filterString.isEmpty() ? null : filterString.charAt(0);
  }

  /** Obtener un criterio a partir de un filtro de caracteres. */
  Criterion getCharacterCriterion() {
    OpCharacter operadorCaracteres = (OpCharacter) filterOp;
    switch (operadorCaracteres) {
      case EMPTY:
        return Restrictions.isNull(filterCutProperty);
      case EQUAL:
        if (filterData != null) return Restrictions.eq(filterCutProperty, filterData);
      default:
        return null;
    }
  }

  // ------------------------------------------------------------------------------------------------- Filtros booleanos

  /** Valores posibles para verdadero. */
  public final static String[] TRUE_VALUES = { "t", "v", "s", "y", "si", "yes", "true", "verdadero" };

  /** Valores posibles para falsos. */
  public final static String[] FALSE_VALUES = { "f", "n", "no", "false", "falso" };

  /** Parsea un filtro para una propiedad de tipo booleano. */
  private void parseBoolean() {
    // Parsear filterOp
    String filterString = this.filterString.trim();
    if (filterString == null || filterString.isEmpty() || filterString.startsWith("!")) {
      this.filterOp = OpBoolean.EMPTY;
      return;
    }
    if (filterString.startsWith("=")) filterString = filterString.substring(1).trim();

    // Mirar haber si coincide con los formatos posibles para falso
    this.filterOp = OpBoolean.EQUAL;
    for (String valor : TRUE_VALUES)
      if (filterString.equalsIgnoreCase(valor)) {
        this.filterData = true;
        return;
      }

    // Mirar haber si coincide con los formatos posibles para falso
    for (String valor : FALSE_VALUES)
      if (filterString.equalsIgnoreCase(valor)) {
        this.filterData = false;
        return;
      }

    // Intentarlo de forma numérica
    try {
      Number numero = NumberFormat.getNumberInstance().parse(filterString);
      if (numero.intValue() == 0) this.filterData = false;
      else this.filterData = true;
    } catch (ParseException e) {
      // Ha habido problemas: entonces el filterData a nulo
      this.filterData = null;
    }

  }

  /** Obtener un criterio de un filtro booleano. */
  Criterion getBooleanCriterion() {
    OpBoolean op = (OpBoolean) filterOp;
    switch (op) {
      case EMPTY:
        return Restrictions.isNull(filterCutProperty);
      case EQUAL:
        if (filterData != null) return Restrictions.eq(filterCutProperty, filterData);
      default:
        return null;
    }
  }

}
