package es.csa.commons.persistence.service;

/*
 * #%L
 * Librerías comunes de Java Server Faces de CSA
 * %%
 * Copyright (C) 2015 CSA - Centro Regional de Servicios Avanzados S.A.
 * %%
 * This file is part of "Librerías comunes de CSA".
 * 
 * developed by: Centro Regional de Servicios Avanzados, S.A.
 * 
 * "Librerías comunes de CSA" is free software: you can redistribute it and/or modify it under the terms of:
 * - Apache License, Version 2.0 (the "License");  
 * - under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 * 
 * "Librerías comunes de CSA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or General Public License for more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE­2.0
 * 
 * You may obtain a copy of the General Public Licence at:
 * http://www.gnu.org/licenses/
 * #L%
 */
 
import java.io.Serializable;
import java.lang.reflect.ParameterizedType;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

import org.springframework.transaction.annotation.Transactional;

public abstract class AbstractServiceImpl<E, ID extends Serializable> implements Serializable, AbstractService<E, ID> {

  private static final long serialVersionUID = -3111698498934622482L;

  private transient EntityManager em;

  private Class<E> entityClass;

  public AbstractServiceImpl() {
    try {
      ParameterizedType genericSuperclass = (ParameterizedType) getClass().getGenericSuperclass();
      @SuppressWarnings("unchecked")
      Class<E> entityClass = (Class<E>) genericSuperclass.getActualTypeArguments()[0];
      this.entityClass = entityClass;
    } catch (Exception e) {
      // Si no se especifica la E, que se pueda utilizar especificando el entityClass en cada método.
      // El resto de métodos devolverán UnsupportedOperationException
    }
  }

  @Override
  public EntityManager getEntityManager() {
    return em;
  }

  @PersistenceContext
  public void setEntityManager(EntityManager em) {
    this.em = em;
  }

  @Override
  @Transactional(readOnly = false)
  public <T> void persist(T entity) {
    em.persist(entity);
  }

  @Override
  @Transactional(readOnly = false)
  public <T> void remove(T entity) {
    em.remove(entity);
  }

  @Override
  @Transactional(readOnly = false)
  public <T> T merge(T entity) {
    T mergedEntity = em.merge(entity);
    return mergedEntity;
  }

  @Override
  @Transactional(readOnly = true)
  public E find(ID id) {
    if (entityClass == null) throw new UnsupportedOperationException("EntityClass is null");
    return em.find(entityClass, id);
  }

  @Override
  @Transactional(readOnly = true)
  public <T, K extends Serializable> T find(Class<T> entityClass, K id) {
    return em.find(entityClass, id);
  }

  @Override
  @SuppressWarnings("unchecked")
  @Transactional(readOnly = true)
  public List<E> findAll() {
    if (entityClass == null) throw new UnsupportedOperationException("EntityClass is null");
    List<E> resultList = generateQuery(entityClass, null, null, false).getResultList();
    return resultList;
  }

  @Override
  @SuppressWarnings("unchecked")
  @Transactional(readOnly = true)
  public <T> List<T> findAll(Class<T> entityClass) {
    List<T> resultList = generateQuery(entityClass, null, null, false).getResultList();
    return resultList;
  }

  @Override
  @Transactional(readOnly = true)
  @SuppressWarnings("unchecked")
  public List<E> findAllOrder(String orden, boolean ascendente) {
    if (entityClass == null) throw new UnsupportedOperationException("EntityClass is null");
    List<E> resultList = generateQuery(entityClass, null, orden, ascendente).getResultList();
    return resultList;
  }

  @Override
  @Transactional(readOnly = true)
  @SuppressWarnings("unchecked")
  public <T> List<T> findAllOrder(Class<T> entityClass, String orden, boolean ascendente) {
    List<T> resultList = generateQuery(entityClass, null, orden, ascendente).getResultList();
    return resultList;

  }

  @Override
  @Transactional(readOnly = true)
  @SuppressWarnings("unchecked")
  public E findOneWhere(String propiedad, Object valor) {
    if (entityClass == null) throw new UnsupportedOperationException("EntityClass is null");
    E entity = (E) generateQuery(entityClass, propiedad, valor, null, false).getSingleResult();
    return entity;

  }

  @Override
  @Transactional(readOnly = true)
  @SuppressWarnings("unchecked")
  public <T> T findOneWhere(Class<T> entityClass, String propiedad, Object valor) {
    T entity = (T) generateQuery(entityClass, propiedad, valor, null, false).getSingleResult();
    return entity;
  }

  @Override
  @Transactional(readOnly = true)
  @SuppressWarnings("unchecked")
  public E findOneWhere(Map<String, Object> propiedades) {
    if (entityClass == null) throw new UnsupportedOperationException("EntityClass is null");
    E entity = (E) generateQuery(entityClass, propiedades, null, false).getSingleResult();
    return entity;
  }

  @Override
  @Transactional(readOnly = true)
  @SuppressWarnings("unchecked")
  public <T> T findOneWhere(Class<T> entityClass, Map<String, Object> propiedades) {
    T entity = (T) generateQuery(entityClass, propiedades, null, false).getSingleResult();
    return entity;
  }

  @Override
  @Transactional(readOnly = true)
  @SuppressWarnings("unchecked")
  public List<E> findWhere(String propiedad, Object valor) {
    if (entityClass == null) throw new UnsupportedOperationException("EntityClass is null");
    List<E> resultList = generateQuery(entityClass, propiedad, valor, null, false).getResultList();
    return resultList;
  }

  @Override
  @Transactional(readOnly = true)
  @SuppressWarnings("unchecked")
  public <T> List<T> findWhere(Class<T> entityClass, String propiedad, Object valor) {
    List<T> resultList = generateQuery(entityClass, propiedad, valor, null, false).getResultList();
    return resultList;

  }

  @Override
  @Transactional(readOnly = true)
  @SuppressWarnings("unchecked")
  public List<E> findWhere(Map<String, Object> propiedades) {
    if (entityClass == null) throw new UnsupportedOperationException("EntityClass is null");
    List<E> resultList = generateQuery(entityClass, propiedades, null, false).getResultList();
    return resultList;
  }

  @Override
  @Transactional(readOnly = true)
  @SuppressWarnings("unchecked")
  public <T> List<T> findWhere(Class<T> entityClass, Map<String, Object> propiedades) {
    List<T> resultList = generateQuery(entityClass, propiedades, null, false).getResultList();
    return resultList;
  }

  @Override
  @Transactional(readOnly = true)
  @SuppressWarnings("unchecked")
  public List<E> findWhereOrder(String propiedad, Object valor, String orden, boolean ascendente) {
    if (entityClass == null) throw new UnsupportedOperationException("EntityClass is null");
    List<E> resultList = generateQuery(entityClass, propiedad, valor, orden, false).getResultList();
    return resultList;
  }

  @Override
  @Transactional(readOnly = true)
  @SuppressWarnings("unchecked")
  public <T> List<T> findWhereOrder(Class<T> entityClass, String propiedad, Object valor, String orden,
      boolean ascendente) {
    List<T> resultList = generateQuery(entityClass, propiedad, valor, orden, false).getResultList();
    return resultList;

  }

  @Override
  @Transactional(readOnly = true)
  @SuppressWarnings("unchecked")
  public List<E> findWhereOrder(Map<String, Object> propiedades, String orden, boolean ascendente) {
    if (entityClass == null) throw new UnsupportedOperationException("EntityClass is null");
    List<E> resultList = generateQuery(entityClass, propiedades, orden, false).getResultList();
    return resultList;
  }

  @Override
  @Transactional(readOnly = true)
  @SuppressWarnings("unchecked")
  public <T> List<T> findWhereOrder(Class<T> entityClass, Map<String, Object> propiedades, String orden,
      boolean ascendente) {
    List<T> resultList = generateQuery(entityClass, propiedades, orden, false).getResultList();
    return resultList;
  }

  @Override
  @Transactional(readOnly = true)
  public List<E> findByQuery(String strQuery) {

    return findByQuery(strQuery, null);
  }

  @Override
  @Transactional(readOnly = true)
  public <T> List<T> findByQuery(Class<T> entitClass, String strQuery) {
    return findByQuery(entitClass, strQuery, null);
  }

  @Override
  @Transactional(readOnly = true)
  @SuppressWarnings("unchecked")
  public List<E> findByQuery(String strQuery, Map<String, ? extends Object> params) {
    if (entityClass == null) throw new UnsupportedOperationException("EntityClass is null");
    Query query;
    query = em.createQuery(strQuery);
    if (params != null) for (Map.Entry<String, ? extends Object> entry : params.entrySet())
      query.setParameter(entry.getKey(), entry.getValue());
    List<E> resultList = query.getResultList();
    return resultList;
  }

  @Override
  @Transactional(readOnly = true)
  @SuppressWarnings("unchecked")
  public <T> List<T> findByQuery(Class<T> entityClass, String strQuery, Map<String, ? extends Object> params) {
    Query query;
    query = em.createQuery(strQuery);
    if (params != null) for (Map.Entry<String, ? extends Object> entry : params.entrySet())
      query.setParameter(entry.getKey(), entry.getValue());

    List<T> resultList = query.getResultList();
    return resultList;

  }

  @Override
  @Transactional(readOnly = true)
  public List<E> findByNamedQuery(String queryName) {
    if (entityClass == null) throw new UnsupportedOperationException("EntityClass is null");
    return findByNamedQuery(queryName, null);
  }

  @Transactional(readOnly = true)
  public <T> List<T> findByNamedQuery(Class<T> entityClass, String queryName) {
    return findByNamedQuery(entityClass, queryName, null);
  }

  @Override
  @Transactional(readOnly = true)
  @SuppressWarnings("unchecked")
  public List<E> findByNamedQuery(String queryName, Map<String, ? extends Object> params) {
    if (entityClass == null) throw new UnsupportedOperationException("EntityClass is null");
    Query query = em.createNamedQuery(queryName);
    if (params != null) for (Map.Entry<String, ? extends Object> entry : params.entrySet())
      query.setParameter(entry.getKey(), entry.getValue());
    List<E> resultList = query.getResultList();
    return resultList;

  }

  @Override
  @Transactional(readOnly = true)
  @SuppressWarnings("unchecked")
  public <T> List<T> findByNamedQuery(Class<T> entityClass, String queryName, Map<String, ? extends Object> params) {
    Query query = em.createNamedQuery(queryName);
    if (params != null) for (Map.Entry<String, ? extends Object> entry : params.entrySet())
      query.setParameter(entry.getKey(), entry.getValue());
    List<T> resultList = query.getResultList();
    return resultList;
  }

  @Override
  @Transactional(readOnly = true)
  public <T> boolean contains(T entity) {
    return em.contains(entity);
  }

  @Override
  @Transactional(readOnly = true)
  public <T> void refresh(T entity) {
    em.refresh(entity);
  }

  @Override
  @Transactional(readOnly = true)
  public <T> void clear() {
    em.clear();
  }

  @Override
  @Transactional
  public <T> void persistAll(Collection<T> entities) {
    for (T entity : entities)
      em.persist(entity);
  }

  @Override
  @Transactional
  public <T> void persistAll(T... entities) {
    persistAll(Arrays.asList(entities));
  }

  @Override
  @Transactional
  public <T> Collection<T> mergeAll(Collection<T> entitites) {
    List<T> mergeds = new ArrayList<T>();
    for (T entity : entitites)
      mergeds.add(em.merge(entity));
    return mergeds;
  }

  @Override
  @Transactional
  public <T> Collection<T> mergeAll(T... entities) {
    return mergeAll(Arrays.asList(entities));
  }

  @Override
  @Transactional
  public <T> void removeAll(Collection<T> entities) {
    for (T entity : entities)
      em.remove(entity);
  }

  @Override
  @Transactional
  public <T> void removeAll(Class<T> tipoEntidad, T... entities) {
    removeAll(Arrays.asList(entities));
  }

  @Override
  @Transactional
  public int executeUpdate(String strQuery) {
    return executeUpdate(strQuery, null);
  }

  @Override
  @Transactional
  public int executeUpdate(String strQuery, Map<String, ?> params) {
    int number;
    Query query;
    query = em.createQuery(strQuery);
    if (params != null) for (Map.Entry<String, ? extends Object> entry : params.entrySet())
      query.setParameter(entry.getKey(), entry.getValue());
    number = query.executeUpdate();
    return number;
  }

  @Override
  @Transactional
  public int executeNamedUpdate(String queryName) {
    return executeNamedUpdate(queryName, null);
  }

  @Override
  @Transactional
  public int executeNamedUpdate(String queryName, Map<String, ?> params) {
    int number;
    Query query;
    query = em.createNamedQuery(queryName);
    if (params != null) for (Map.Entry<String, ? extends Object> entry : params.entrySet())
      query.setParameter(entry.getKey(), entry.getValue());
    number = query.executeUpdate();
    return number;
  }

  /**
   * Método auxiliar para generar consultas con varias condiciones.
   * 
   * @param properties
   *          Las propiedades de la condición.
   * @param order
   *          El nombre de la propiedad sobre la que orden. Nulo indica sin orden específico.
   * @param ascending
   *          Si es verdadero se ordena de forma ascendente, sino, de forma descendente.
   * @param <T>
   *          El tipo de la entidad.
   * @return La consulta.
   */
  @Transactional
  protected <T> Query generateQuery(Class<T> entityClass, Map<String, Object> properties, String order,
      boolean ascending) {
    String strQuery = "SELECT e FROM {tipo} e";
    List<String> keys = new ArrayList<String>();
    if (properties != null) keys.addAll(properties.keySet());
    for (int i = 0, size = keys.size(); i < size; i++) {
      strQuery += (i == 0 ? " WHERE " : " AND ");
      strQuery += ("e." + keys.get(i) + " = :valor" + i);
    }
    strQuery = strQuery.replace("{tipo}", entityClass.getName());
    if (order != null) {
      strQuery += " ORDER BY {orden} {ascendente}";
      strQuery = strQuery.replace("{orden}", order);
      strQuery = strQuery.replace("{ascendente}", ascending ? "ASC" : "DESC");
    }
    Query query = em.createQuery(strQuery);
    for (int i = 0; i < keys.size(); i++)
      query.setParameter("valor" + i, properties.get(keys.get(i)));
    return query;
  }

  /**
   * Método auxiliar para generar consultas con una condición.
   * 
   * @param tipo
   *          La clase de la entidad sobre la que se consulta.
   * @param property
   *          El nombre de la propiedad.
   * @param value
   *          El valor que se buscará en la propiedad.
   * @param order
   *          El nombre de la propiedad sobre la que ordenar. Nulo indica sin orden específico.
   * @param ascending
   *          Si es verdadero se ordena de forma ascendente, sino, de forma descendente.
   * @param <T>
   *          El tipo de la entidad.
   * @return La consulta.
   */
  @Transactional
  protected <T> Query generateQuery(Class<T> entityClass, String property, Object value, String order, boolean ascending) {
    Map<String, Object> propiedades = new HashMap<String, Object>();
    propiedades.put(property, value);
    return generateQuery(entityClass, propiedades, order, ascending);
  }

}
