package es.csa.fdt.cyldigital.centros.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.StringTokenizer;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.annotation.Resource;

import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.fdt.cyldigital.centros.entity.Aula;
import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap;


/**
 * Servicio de Aulas.
 * @author csa
 *
 */
@Service
@Repository
public class AulaServiceImpl extends AbstractServiceImpl<Aula, Long> implements AulaService {

  private List<Centro> centros;

  private static final long serialVersionUID = -5575228180361917674L;

  private static final String IPADDRESS_PATTERN = "^([01]?\\d\\d?|2[0-4]\\d|25[0-5])\\."
      + "([01]?\\d\\d?|2[0-4]\\d|25[0-5])\\." + "([01]?\\d\\d?|2[0-4]\\d|25[0-5])\\."
      + "([01]?\\d\\d?|2[0-4]\\d|25[0-5])$";

  @Resource
  private transient HashMap<String, Object> generalProperties;

  /**
   * Construye un nuevo objeto de la clase
   */
  public AulaServiceImpl() {
    super();
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.centros.service.AulaService#persistAula(es.csa.fdt.cyldigital.centros.entity.Aula)
   */
  public boolean persistAula(Aula aula) {
    if (aula.getIp() != null && !aula.getIp().isEmpty() && aula.getMask().isEmpty()) {
      addErrorMessage("Se debe especificar la mascara de red");
      return false;
    }
    if (aula.getIp() != null && !aula.getIp().isEmpty() && !isValidIp(aula.getIp())) {
      addErrorMessage("La dirección ip especificada no es valida");
      return false;
    }
    if (aula.getIp() != null && !aula.getIp().isEmpty() && !isValidMask(aula.getMask())) {
      addErrorMessage("La mascara especificada no es valida");
      return false;
    }
    persist(aula);

    // si ha cambiado la localización por defecto, se actualizan las otras areas
    if (aula.isLocalizacionPorDefecto()) {
      List<Aula> aulas = findWhere("centro", aula.getCentro());
      for (Aula a : aulas) {
        if (a != aula) {
          disableMessages();
          a.setLocalizacionPorDefecto(false);
          persist(a);
        }
      }
    }

    return true;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.centros.service.AulaService#getAulasDisponibles()
   */
  public List<Aula> getAulasDisponibles() {
    return findAll(Aula.class);
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.centros.service.AulaService#getCentros()
   */
  public List<Centro> getCentros() {
    if (centros == null) centros = findAllOrder(Centro.class, "nombre", true);
    return centros;
  }

  /**
   * Actualiza centro.
   * 
   * @param aula
   *           aula
   * @param currentUser
   *           current user
   */
  public void actualizaCentro(Aula aula, UsuarioLdap currentUser) {
    if (currentUser.applyFilterCentro() && currentUser.getCentro() != null) {
      if (centros == null) {
        centros = new ArrayList<Centro>();
      }
      Centro c = find(Centro.class, currentUser.getCentro().getId());
      aula.setCentro(c);
      centros.add(c);
    } else if (generalProperties.containsKey("DEFAULT_CENTER_ID") && aula.getCentro() == null) {
      Centro c = find(Centro.class, Long.valueOf((String) generalProperties.get("DEFAULT_CENTER_ID")));
      if (c != null) aula.setCentro(c);
    }
  }

  /**
   * Find aula.
   * 
   * @param idAula
   *           id aula
   * @return aula
   */
  public Aula findAula(Long idAula) {
    return idAula != null ? find(idAula) : null;
  }

  private boolean isValidMask(String ip) {
    int[] direccion = splitIp(ip);
    int[] valid = { 0, 128, 192, 224, 240, 248, 252, 254, 255 };
    boolean flag = false;

    for (int i : direccion) {
      flag = false;
      for (int j : valid) {
        if (j == i) flag = true;
      }
      if (!flag) return false;
    }
    return true;
  }

  private boolean isValidIp(String ip) {
    Pattern pattern = Pattern.compile(IPADDRESS_PATTERN);
    Matcher matcher = pattern.matcher(ip);
    return matcher.matches();
  }

  private static int[] splitIp(String ip) {
    int[] result = new int[4];
    StringTokenizer st = new StringTokenizer(ip, ".");
    for (int i = 0; i < 4; i++) {
      result[i] = Integer.parseInt(st.nextToken());
    }
    return result;
  }
}
