package es.csa.fdt.cyldigital.core.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.lang.reflect.ParameterizedType;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

import org.springframework.binding.message.MessageBuilder;
import org.springframework.binding.message.MessageContext;
import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.webflow.execution.RequestContextHolder;

/**
 * Clase  AbstractServiceImpl.
 * 
 * @author csa
 * 
 * @param <E>
 *           element type
 * @param <ID>
 *           generic type
 */
@Service
@Repository
public abstract class AbstractServiceImpl<E, ID extends Serializable> implements Serializable, AbstractService<E, ID> {

  private static final long serialVersionUID = -3111698498934622482L;

  private transient EntityManager em;

  private Class<E> entityClass;

  private boolean messagesEnabled;

  /**
   * Construye un nuevo objeto de la clase
   */
  public AbstractServiceImpl() {
    setMessagesEnabled(true);
    try {
      ParameterizedType genericSuperclass = (ParameterizedType) getClass().getGenericSuperclass();
      @SuppressWarnings("unchecked")
      Class<E> entityClass = (Class<E>) genericSuperclass.getActualTypeArguments()[0];
      this.entityClass = entityClass;
    } catch (Exception e) {
      // Si no se especifica la E, que se pueda utilizar especificando el entityClass en cada método.
      // El resto de métodos devolverán UnsupportedOperationException
    }
  }

  /*
   * (non-Javadoc)
   * 
   * @see es.csa.fdt.cyldigital.core.service.AbstractService#getEntityManager()
   */
  @Override
  public EntityManager getEntityManager() {
    return em;
  }

  /**
   * Asigna entity manager.
   * 
   * @param em
   *          entity manager
   */
  @PersistenceContext
  public void setEntityManager(EntityManager em) {
    this.em = em;
  }

  /**
   * Checks if is messages enabled.
   * 
   * @return true, if is messages enabled
   */
  public boolean isMessagesEnabled() {
    return messagesEnabled;
  }

  /**
   * Asigna messages enabled.
   * 
   * @param enabled
   *          messages enabled
   */
  public void setMessagesEnabled(boolean enabled) {
    this.messagesEnabled = enabled;
  }

  /**
   * Enable messages.
   */
  public void enableMessages() {
    setMessagesEnabled(true);
  }

  /**
   * Disable messages.
   */
  public void disableMessages() {
    setMessagesEnabled(false);
  }

  /*
   * (non-Javadoc)
   * 
   * @see es.csa.fdt.cyldigital.core.service.AbstractService#persist(java.lang.Object)
   */
  @Override
  @Transactional(readOnly = false)
  public <T> void persist(T entity) {
    em.persist(entity);
    addInfoMessage("service.persist.ok");
  }

  /*
   * (non-Javadoc)
   * 
   * @see es.csa.fdt.cyldigital.core.service.AbstractService#remove(java.lang.Object)
   */
  @Override
  @Transactional(readOnly = false)
  public <T> void remove(T entity) {
    em.remove(entity);
    addInfoMessage("service.remove.ok");
  }

  /*
   * (non-Javadoc)
   * 
   * @see es.csa.fdt.cyldigital.core.service.AbstractService#merge(java.lang.Object)
   */
  @Override
  @Transactional(readOnly = false)
  public <T> T merge(T entity) {
    T mergedEntity = em.merge(entity);
    addInfoMessage("service.merge.ok");
    return mergedEntity;
  }

  /*
   * (non-Javadoc)
   * 
   * @see es.csa.fdt.cyldigital.core.service.AbstractService#find(java.io.Serializable)
   */
  @Override
  @Transactional(readOnly = true)
  public E find(ID id) {
    if (entityClass == null) throw new UnsupportedOperationException("EntityClass is null");
    return em.find(entityClass, id);
  }

  /*
   * (non-Javadoc)
   * 
   * @see es.csa.fdt.cyldigital.core.service.AbstractService#find(java.lang.Class, java.io.Serializable)
   */
  @Override
  @Transactional(readOnly = true)
  public <T, K extends Serializable> T find(Class<T> entityClass, K id) {
    return em.find(entityClass, id);
  }

  /*
   * (non-Javadoc)
   * 
   * @see es.csa.fdt.cyldigital.core.service.AbstractService#findAll()
   */
  @Override
  @SuppressWarnings("unchecked")
  @Transactional(readOnly = true)
  public List<E> findAll() {
    if (entityClass == null) throw new UnsupportedOperationException("EntityClass is null");
    List<E> resultList = generateQuery(entityClass, null, null, false).getResultList();
    return resultList;
  }

  /*
   * (non-Javadoc)
   * 
   * @see es.csa.fdt.cyldigital.core.service.AbstractService#findAll(java.lang.Class)
   */
  @Override
  @SuppressWarnings("unchecked")
  @Transactional(readOnly = true)
  public <T> List<T> findAll(Class<T> entityClass) {
    List<T> resultList = generateQuery(entityClass, null, null, false).getResultList();
    return resultList;
  }

  /*
   * (non-Javadoc)
   * 
   * @see es.csa.fdt.cyldigital.core.service.AbstractService#findAllOrder(java.lang.String, boolean)
   */
  @Override
  @Transactional(readOnly = true)
  @SuppressWarnings("unchecked")
  public List<E> findAllOrder(String orden, boolean ascendente) {
    if (entityClass == null) throw new UnsupportedOperationException("EntityClass is null");
    List<E> resultList = generateQuery(entityClass, null, orden, ascendente).getResultList();
    return resultList;
  }

  /*
   * (non-Javadoc)
   * 
   * @see es.csa.fdt.cyldigital.core.service.AbstractService#findAllOrder(java.lang.Class, java.lang.String, boolean)
   */
  @Override
  @Transactional(readOnly = true)
  @SuppressWarnings("unchecked")
  public <T> List<T> findAllOrder(Class<T> entityClass, String orden, boolean ascendente) {
    List<T> resultList = generateQuery(entityClass, null, orden, ascendente).getResultList();
    return resultList;

  }

  /*
   * (non-Javadoc)
   * 
   * @see es.csa.fdt.cyldigital.core.service.AbstractService#findOneWhere(java.lang.String, java.lang.Object)
   */
  @Override
  @Transactional(readOnly = true)
  @SuppressWarnings("unchecked")
  public E findOneWhere(String propiedad, Object valor) {
    if (entityClass == null) throw new UnsupportedOperationException("EntityClass is null");
    E entity = (E) generateQuery(entityClass, propiedad, valor, null, false).getSingleResult();
    return entity;

  }

  /*
   * (non-Javadoc)
   * 
   * @see es.csa.fdt.cyldigital.core.service.AbstractService#findOneWhere(java.lang.Class, java.lang.String,
   * java.lang.Object)
   */
  @Override
  @Transactional(readOnly = true)
  @SuppressWarnings("unchecked")
  public <T> T findOneWhere(Class<T> entityClass, String propiedad, Object valor) {
    T entity = (T) generateQuery(entityClass, propiedad, valor, null, false).getSingleResult();
    return entity;
  }

  /*
   * (non-Javadoc)
   * 
   * @see es.csa.fdt.cyldigital.core.service.AbstractService#findOneWhere(java.util.Map)
   */
  @Override
  @Transactional(readOnly = true)
  @SuppressWarnings("unchecked")
  public E findOneWhere(Map<String, Object> propiedades) {
    if (entityClass == null) throw new UnsupportedOperationException("EntityClass is null");
    E entity = (E) generateQuery(entityClass, propiedades, null, false).getSingleResult();
    return entity;
  }

  /*
   * (non-Javadoc)
   * 
   * @see es.csa.fdt.cyldigital.core.service.AbstractService#findOneWhere(java.lang.Class, java.util.Map)
   */
  @Override
  @Transactional(readOnly = true)
  @SuppressWarnings("unchecked")
  public <T> T findOneWhere(Class<T> entityClass, Map<String, Object> propiedades) {
    T entity = (T) generateQuery(entityClass, propiedades, null, false).getSingleResult();
    return entity;
  }

  /*
   * (non-Javadoc)
   * 
   * @see es.csa.fdt.cyldigital.core.service.AbstractService#findWhere(java.lang.String, java.lang.Object)
   */
  @Override
  @Transactional(readOnly = true)
  @SuppressWarnings("unchecked")
  public List<E> findWhere(String propiedad, Object valor) {
    if (entityClass == null) throw new UnsupportedOperationException("EntityClass is null");
    List<E> resultList = generateQuery(entityClass, propiedad, valor, null, false).getResultList();
    return resultList;
  }

  /*
   * (non-Javadoc)
   * 
   * @see es.csa.fdt.cyldigital.core.service.AbstractService#findWhere(java.lang.Class, java.lang.String,
   * java.lang.Object)
   */
  @Override
  @Transactional(readOnly = true)
  @SuppressWarnings("unchecked")
  public <T> List<T> findWhere(Class<T> entityClass, String propiedad, Object valor) {
    List<T> resultList = generateQuery(entityClass, propiedad, valor, null, false).getResultList();
    return resultList;

  }

  /*
   * (non-Javadoc)
   * 
   * @see es.csa.fdt.cyldigital.core.service.AbstractService#findWhere(java.util.Map)
   */
  @Override
  @Transactional(readOnly = true)
  @SuppressWarnings("unchecked")
  public List<E> findWhere(Map<String, Object> propiedades) {
    if (entityClass == null) throw new UnsupportedOperationException("EntityClass is null");
    List<E> resultList = generateQuery(entityClass, propiedades, null, false).getResultList();
    return resultList;
  }

  /*
   * (non-Javadoc)
   * 
   * @see es.csa.fdt.cyldigital.core.service.AbstractService#findWhere(java.lang.Class, java.util.Map)
   */
  @Override
  @Transactional(readOnly = true)
  @SuppressWarnings("unchecked")
  public <T> List<T> findWhere(Class<T> entityClass, Map<String, Object> propiedades) {
    List<T> resultList = generateQuery(entityClass, propiedades, null, false).getResultList();
    return resultList;
  }

  /*
   * (non-Javadoc)
   * 
   * @see es.csa.fdt.cyldigital.core.service.AbstractService#findWhereOrder(java.lang.String, java.lang.Object,
   * java.lang.String, boolean)
   */
  @Override
  @Transactional(readOnly = true)
  @SuppressWarnings("unchecked")
  public List<E> findWhereOrder(String propiedad, Object valor, String orden, boolean ascendente) {
    if (entityClass == null) throw new UnsupportedOperationException("EntityClass is null");
    List<E> resultList = generateQuery(entityClass, propiedad, valor, orden, ascendente).getResultList();
    return resultList;
  }

  /*
   * (non-Javadoc)
   * 
   * @see es.csa.fdt.cyldigital.core.service.AbstractService#findWhereOrder(java.lang.Class, java.lang.String,
   * java.lang.Object, java.lang.String, boolean)
   */
  @Override
  @Transactional(readOnly = true)
  @SuppressWarnings("unchecked")
  public <T> List<T> findWhereOrder(Class<T> entityClass, String propiedad, Object valor, String orden,
      boolean ascendente) {
    List<T> resultList = generateQuery(entityClass, propiedad, valor, orden, ascendente).getResultList();
    return resultList;

  }

  /*
   * (non-Javadoc)
   * 
   * @see es.csa.fdt.cyldigital.core.service.AbstractService#findWhereOrder(java.util.Map, java.lang.String, boolean)
   */
  @Override
  @Transactional(readOnly = true)
  @SuppressWarnings("unchecked")
  public List<E> findWhereOrder(Map<String, Object> propiedades, String orden, boolean ascendente) {
    if (entityClass == null) throw new UnsupportedOperationException("EntityClass is null");
    List<E> resultList = generateQuery(entityClass, propiedades, orden, ascendente).getResultList();
    return resultList;
  }

  /*
   * (non-Javadoc)
   * 
   * @see es.csa.fdt.cyldigital.core.service.AbstractService#findWhereOrder(java.lang.Class, java.util.Map,
   * java.lang.String, boolean)
   */
  @Override
  @Transactional(readOnly = true)
  @SuppressWarnings("unchecked")
  public <T> List<T> findWhereOrder(Class<T> entityClass, Map<String, Object> propiedades, String orden,
      boolean ascendente) {
    List<T> resultList = generateQuery(entityClass, propiedades, orden, ascendente).getResultList();
    return resultList;
  }

  /*
   * (non-Javadoc)
   * 
   * @see es.csa.fdt.cyldigital.core.service.AbstractService#findByQuery(java.lang.String)
   */
  @Override
  @Transactional(readOnly = true)
  public List<E> findByQuery(String strQuery) {

    return findByQuery(strQuery, null);
  }

  /*
   * (non-Javadoc)
   * 
   * @see es.csa.fdt.cyldigital.core.service.AbstractService#findByQuery(java.lang.Class, java.lang.String)
   */
  @Override
  @Transactional(readOnly = true)
  public <T> List<T> findByQuery(Class<T> entitClass, String strQuery) {
    return findByQuery(entitClass, strQuery, null);
  }

  /*
   * (non-Javadoc)
   * 
   * @see es.csa.fdt.cyldigital.core.service.AbstractService#findByQuery(java.lang.String, java.util.Map)
   */
  @Override
  @Transactional(readOnly = true)
  @SuppressWarnings("unchecked")
  public List<E> findByQuery(String strQuery, Map<String, ? extends Object> params) {
    if (entityClass == null) throw new UnsupportedOperationException("EntityClass is null");
    Query query;
    query = em.createQuery(strQuery);
    if (params != null) for (Map.Entry<String, ? extends Object> entry : params.entrySet())
      query.setParameter(entry.getKey(), entry.getValue());
    List<E> resultList = query.getResultList();
    return resultList;
  }

  /*
   * (non-Javadoc)
   * 
   * @see es.csa.fdt.cyldigital.core.service.AbstractService#findByQuery(java.lang.Class, java.lang.String,
   * java.util.Map)
   */
  @Override
  @Transactional(readOnly = true)
  @SuppressWarnings("unchecked")
  public <T> List<T> findByQuery(Class<T> entityClass, String strQuery, Map<String, ? extends Object> params) {
    Query query;
    query = em.createQuery(strQuery);
    if (params != null) for (Map.Entry<String, ? extends Object> entry : params.entrySet())
      query.setParameter(entry.getKey(), entry.getValue());

    List<T> resultList = query.getResultList();
    return resultList;

  }

  /*
   * (non-Javadoc)
   * 
   * @see es.csa.fdt.cyldigital.core.service.AbstractService#findByNamedQuery(java.lang.String)
   */
  @Override
  @Transactional(readOnly = true)
  public List<E> findByNamedQuery(String queryName) {
    if (entityClass == null) throw new UnsupportedOperationException("EntityClass is null");
    return findByNamedQuery(queryName, null);
  }

  /*
   * (non-Javadoc)
   * 
   * @see es.csa.fdt.cyldigital.core.service.AbstractService#findByNamedQuery(java.lang.Class, java.lang.String)
   */
  @Transactional(readOnly = true)
  public <T> List<T> findByNamedQuery(Class<T> entityClass, String queryName) {
    return findByNamedQuery(entityClass, queryName, null);
  }

  /*
   * (non-Javadoc)
   * 
   * @see es.csa.fdt.cyldigital.core.service.AbstractService#findByNamedQuery(java.lang.String, java.util.Map)
   */
  @Override
  @Transactional(readOnly = true)
  @SuppressWarnings("unchecked")
  public List<E> findByNamedQuery(String queryName, Map<String, ? extends Object> params) {
    if (entityClass == null) throw new UnsupportedOperationException("EntityClass is null");
    Query query = em.createNamedQuery(queryName);
    if (params != null) for (Map.Entry<String, ? extends Object> entry : params.entrySet())
      query.setParameter(entry.getKey(), entry.getValue());
    List<E> resultList = query.getResultList();
    return resultList;

  }

  /*
   * (non-Javadoc)
   * 
   * @see es.csa.fdt.cyldigital.core.service.AbstractService#findByNamedQuery(java.lang.Class, java.lang.String,
   * java.util.Map)
   */
  @Override
  @Transactional(readOnly = true)
  @SuppressWarnings("unchecked")
  public <T> List<T> findByNamedQuery(Class<T> entityClass, String queryName, Map<String, ? extends Object> params) {
    Query query = em.createNamedQuery(queryName);
    if (params != null) for (Map.Entry<String, ? extends Object> entry : params.entrySet())
      query.setParameter(entry.getKey(), entry.getValue());
    List<T> resultList = query.getResultList();
    return resultList;
  }

  /*
   * (non-Javadoc)
   * 
   * @see es.csa.fdt.cyldigital.core.service.AbstractService#contains(java.lang.Object)
   */
  @Override
  @Transactional(readOnly = true)
  public <T> boolean contains(T entity) {
    return em.contains(entity);
  }

  /*
   * (non-Javadoc)
   * 
   * @see es.csa.fdt.cyldigital.core.service.AbstractService#refresh(java.lang.Object)
   */
  @Override
  @Transactional(readOnly = true)
  public <T> void refresh(T entity) {
    em.refresh(entity);
  }

  /*
   * (non-Javadoc)
   * 
   * @see es.csa.fdt.cyldigital.core.service.AbstractService#clear()
   */
  @Override
  @Transactional(readOnly = true)
  public <T> void clear() {
    em.clear();
  }

  /*
   * (non-Javadoc)
   * 
   * @see es.csa.fdt.cyldigital.core.service.AbstractService#persistAll(java.util.Collection)
   */
  @Override
  @Transactional
  public <T> void persistAll(Collection<T> entities) {
    for (T entity : entities)
      em.persist(entity);
    addInfoMessage("service.persistAll.ok");
  }

  /*
   * (non-Javadoc)
   * 
   * @see es.csa.fdt.cyldigital.core.service.AbstractService#persistAll(T[])
   */
  @Override
  @Transactional
  public <T> void persistAll(T... entities) {
    persistAll(Arrays.asList(entities));
  }

  /*
   * (non-Javadoc)
   * 
   * @see es.csa.fdt.cyldigital.core.service.AbstractService#mergeAll(java.util.Collection)
   */
  @Override
  @Transactional
  public <T> Collection<T> mergeAll(Collection<T> entitites) {
    List<T> mergeds = new ArrayList<T>();
    for (T entity : entitites)
      mergeds.add(em.merge(entity));
    addInfoMessage("service.mergeAll.ok");
    return mergeds;
  }

  /*
   * (non-Javadoc)
   * 
   * @see es.csa.fdt.cyldigital.core.service.AbstractService#mergeAll(T[])
   */
  @Override
  @Transactional
  public <T> Collection<T> mergeAll(T... entities) {
    return mergeAll(Arrays.asList(entities));
  }

  /*
   * (non-Javadoc)
   * 
   * @see es.csa.fdt.cyldigital.core.service.AbstractService#removeAll(java.util.Collection)
   */
  @Override
  @Transactional
  public <T> void removeAll(Collection<T> entities) {
    for (T entity : entities)
      em.remove(entity);
    addInfoMessage("service.removeAll.ok");
  }

  /*
   * (non-Javadoc)
   * 
   * @see es.csa.fdt.cyldigital.core.service.AbstractService#removeAll(java.lang.Class, T[])
   */
  @Override
  @Transactional
  public <T> void removeAll(Class<T> tipoEntidad, T... entities) {
    removeAll(Arrays.asList(entities));
  }

  /*
   * (non-Javadoc)
   * 
   * @see es.csa.fdt.cyldigital.core.service.AbstractService#executeUpdate(java.lang.String)
   */
  @Override
  @Transactional
  public int executeUpdate(String strQuery) {
    return executeUpdate(strQuery, null);
  }

  /*
   * (non-Javadoc)
   * 
   * @see es.csa.fdt.cyldigital.core.service.AbstractService#executeUpdate(java.lang.String, java.util.Map)
   */
  @Override
  @Transactional
  public int executeUpdate(String strQuery, Map<String, ?> params) {
    int number;
    Query query;
    query = em.createQuery(strQuery);
    if (params != null) for (Map.Entry<String, ? extends Object> entry : params.entrySet())
      query.setParameter(entry.getKey(), entry.getValue());
    number = query.executeUpdate();
    addInfoMessage("service.executeUpdate.ok");
    return number;
  }

  /*
   * (non-Javadoc)
   * 
   * @see es.csa.fdt.cyldigital.core.service.AbstractService#executeNamedUpdate(java.lang.String)
   */
  @Override
  @Transactional
  public int executeNamedUpdate(String queryName) {
    return executeNamedUpdate(queryName, null);
  }

  /*
   * (non-Javadoc)
   * 
   * @see es.csa.fdt.cyldigital.core.service.AbstractService#executeNamedUpdate(java.lang.String, java.util.Map)
   */
  @Override
  @Transactional
  public int executeNamedUpdate(String queryName, Map<String, ?> params) {
    int number;
    Query query;
    query = em.createNamedQuery(queryName);
    if (params != null) for (Map.Entry<String, ? extends Object> entry : params.entrySet())
      query.setParameter(entry.getKey(), entry.getValue());
    number = query.executeUpdate();
    addInfoMessage("service.executeNamedUpdate.ok");
    return number;
  }

  /**
   * Método auxiliar para generar consultas con varias condiciones.
   * 
   * @param properties
   *          Las propiedades de la condición.
   * @param order
   *          El nombre de la propiedad sobre la que orden. Nulo indica sin orden específico.
   * @param ascending
   *          Si es verdadero se ordena de forma ascendente, sino, de forma descendente.
   * @param <T>
   *          El tipo de la entidad.
   * @param entityClass
   *          entityClass
   * @return La consulta.
   */
  @Transactional
  protected <T> Query generateQuery(Class<T> entityClass, Map<String, Object> properties, String order,
      boolean ascending) {
    String strQuery = "SELECT e FROM {tipo} e";
    List<String> keys = new ArrayList<String>();
    if (properties != null) keys.addAll(properties.keySet());
    for (int i = 0, size = keys.size(); i < size; i++) {
      strQuery += (i == 0 ? " WHERE " : " AND ");
      if (properties.get(keys.get(i)) != null)
    	  strQuery += ("e." + keys.get(i) + " = :valor" + i);
      else
    	  strQuery += ("e." + keys.get(i) + " is NULL");
    }
    strQuery = strQuery.replace("{tipo}", entityClass.getName());
    if (order != null) {
      strQuery += " ORDER BY {orden} {ascendente}";
      strQuery = strQuery.replace("{orden}", order);
      strQuery = strQuery.replace("{ascendente}", ascending ? "ASC" : "DESC");
    }
    Query query = em.createQuery(strQuery);
    for (int i = 0; i < keys.size(); i++)
    	if (properties.get(keys.get(i)) != null)
    		query.setParameter("valor" + i, properties.get(keys.get(i)));
    return query;
  }

  /**
   * Método auxiliar para generar consultas con una condición.
   * 
   * @param tipo
   *          La clase de la entidad sobre la que se consulta.
   * @param property
   *          El nombre de la propiedad.
   * @param value
   *          El valor que se buscará en la propiedad.
   * @param order
   *          El nombre de la propiedad sobre la que ordenar. Nulo indica sin orden específico.
   * @param ascending
   *          Si es verdadero se ordena de forma ascendente, sino, de forma descendente.
   * @param <T>
   *          El tipo de la entidad.
   * @param entityClass
   *          entityClass
   * @return La consulta.
   */
  @Transactional
  protected <T> Query generateQuery(Class<T> entityClass, String property, Object value, String order, boolean ascending) {
    Map<String, Object> propiedades = new HashMap<String, Object>();
    propiedades.put(property, value);
    return generateQuery(entityClass, propiedades, order, ascending);
  }

  /**
   * Añade un mensaje de información.
   * 
   * @param code
   *          El código del mensaje.
   */
  protected void addInfoMessage(String code) {
    addInfoMessage(code, null);
  }

  /**
   * Añade info message.
   * 
   * @param code
   *          code
   * @param args
   *          args
   */
  protected void addInfoMessage(String code, Object[] args) {
    try {
      if (isMessagesEnabled()) {
        MessageContext context = RequestContextHolder.getRequestContext().getMessageContext();
        context.addMessage(new MessageBuilder().info().defaultText(code).code(code).args(args).build());
      }
    } catch (NullPointerException e) {
      // TODO: Excepción lanzada cuando se invoca 'setMessagesEnabled(true)' desde un WebService sin contexto de Spring.
    }
  }

  /**
   * Añade un mensaje de warning.
   * 
   * @param code
   *          El código del mensaje.
   */
  protected void addWarnMessage(String code) {
    addWarnMessage(code, null);
  }

  /**
   * Añade warn message.
   * 
   * @param code
   *          code
   * @param args
   *          args
   */
  protected void addWarnMessage(String code, Object[] args) {
    try {
      if (isMessagesEnabled()) {
        MessageContext context = RequestContextHolder.getRequestContext().getMessageContext();
        context.addMessage(new MessageBuilder().warning().defaultText(code).code(code).args(args).build());
      }
    } catch (NullPointerException e) {
      // TODO: Excepción lanzada cuando se invoca 'setMessagesEnabled(true)' desde un WebService sin contexto de Spring.
    }
  }

  /**
   * Añade un mensaje de error.
   * 
   * @param code
   *          El código del mensaje.
   */
  protected void addErrorMessage(String code) {
    addErrorMessage(code, null);
  }

  /**
   * Añade error message.
   * 
   * @param code
   *          code
   * @param args
   *          args
   */
  protected void addErrorMessage(String code, Object[] args) {
    try {
      if (isMessagesEnabled()) {
        MessageContext context = RequestContextHolder.getRequestContext().getMessageContext();
        context.addMessage(new MessageBuilder().error().defaultText(code).code(code).args(args).build());
      }
    } catch (NullPointerException e) {
      // TODO: Excepción lanzada cuando se invoca 'setMessagesEnabled(true)' desde un WebService sin contexto de Spring.
    }
  }

}
