package es.csa.fdt.cyldigital.evaluacion.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * Copyright (C) 2015  Xunta de Galicia <oficinacemit@xunta.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * developed by: Balidea <info@balidea.com>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

import javax.persistence.AttributeOverride;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.annotations.MapKeyManyToMany;
import org.hibernate.envers.Audited;
import org.hibernate.envers.NotAudited;

import es.csa.fdt.cyldigital.core.entity.AbstractEntity;
import es.csa.fdt.cyldigital.evaluacion.entity.Pregunta.TipoPregunta;

/**
 * Represents an instance of a filled form
 * <p>
 * An instance of this class is created when (habitually) a user fill a form.
 * This instance associates the questions of a form with the concrete responses given by the user.
 * <p>
 * On the other hand, this class doesn't known its form template. It works as a question-response pairs stock.
 * Take control about the coherence of questions included in this filled form is a third part responsibility.   
 * 
 * @author csa
 * @author Balidea Consulting & Programming
 */
@Entity
@Table(name = "formulariosconcretos")
@AttributeOverride(name = "id", column = @Column(name = "idFormularioConcreto"))
@Audited
public class FormularioConcreto extends AbstractEntity implements Serializable {

  private static final long serialVersionUID = 50516757113484534L;
  
  /**
   * Stores the responses given by (habitually) a user to the questions associated to the form template
   */
  private Map<Pregunta, Respuesta> responses;

  /**
   * Creates a new filled form instance with a empty collection of questions responses
   */
  public FormularioConcreto() {
    super();
    this.responses = new HashMap<Pregunta, Respuesta>();
  }

  /**
   * Gets the question-response pairs of the filled form
   * 
   * @return the question-response pairs
   */
  @NotAudited
  @ManyToMany(targetEntity = Respuesta.class, cascade = { javax.persistence.CascadeType.ALL })
  @JoinTable(name = "formulariosconcretos_respuestas", joinColumns = @JoinColumn(name = "idFormularioConcreto"), inverseJoinColumns = @JoinColumn(name = "idRespuesta"))
  @MapKeyManyToMany(joinColumns = @JoinColumn(name = "idPregunta"))
  public Map<Pregunta, Respuesta> getRespuestas() {
    return this.responses;
  }

  /**
   * Sets the question-response pairs of the filled form
   * 
   * @param responses
   *           the question-response pairs
   */
  public void setRespuestas(Map<Pregunta, Respuesta> responses) {
    this.responses = responses;
  }

  /**
   * Checks if there are answered questions
   * <p>
   * If there is at least one question answered the questionnaire is considered filled.
   * It doesn't take into account how many questions have the form template neither if questions are mandatory or not.
   * 
   * @return <code>true</code> if there is at least one question answered, <code>false</code> otherwise
   */
  @Transient
  public boolean isFilled() {
	  return this.responses.size() > 0;
  }
  
  /**
   * Calculates the average score of the responses
   * <p>
   * It takes into account only the numeric type questions discarding the rest.
   * It doesn't consider the range of allowed values in the responses assuming coherent ranges for all the questions.
   * If this is not the norm, response values normalization should be implemented: (value - min) / (max - min) 
   * 
   * @return the responses average value for the numeric questions or <code>null</code> if no numeric questions are filled
   */
  @Transient
  public Float getAverageScore() {
	  
	  int counter = 0;
	  float accum = 0f;

	  for (Entry<Pregunta, Respuesta> entry : responses.entrySet()) {
		  
		  TipoPregunta type = entry.getKey().getTipo(); 
		  Respuesta response = entry.getValue();
		  
		  if (type == TipoPregunta.ENTERO || type == TipoPregunta.DECIMAL) {
			  counter++;
			  if (type == TipoPregunta.ENTERO)
				  accum += response.getValorEntero();
			  else
				  accum += response.getValorDecimal();
		  }
	  }
	  
	  Float average = (counter == 0) ? null : accum / counter; 
	  
	  return average;
	  
  }

}
