package es.csa.fdt.cyldigital.usuarios.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.text.MessageFormat;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;

import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.directory.Attributes;
import javax.naming.directory.DirContext;
import javax.naming.directory.SearchControls;
import javax.naming.directory.SearchResult;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.dao.IncorrectResultSizeDataAccessException;
import org.springframework.ldap.PartialResultException;
import org.springframework.ldap.core.ContextExecutor;
import org.springframework.ldap.core.ContextMapper;
import org.springframework.ldap.core.ContextSource;
import org.springframework.ldap.core.DirContextAdapter;
import org.springframework.ldap.core.DirContextOperations;
import org.springframework.ldap.core.DistinguishedName;
import org.springframework.ldap.core.LdapEncoder;
import org.springframework.ldap.core.LdapTemplate;
import org.springframework.security.ldap.SpringSecurityLdapTemplate;
import org.springframework.util.Assert;

/**
 * Extension of Spring LDAP's LdapTemplate class which adds extra functionality required by Spring Security.
 * 
 * @author Ben Alex
 * @author Luke Taylor
 * @since 2.0
 */
public class SpringSecurityLdapTemplateMock extends LdapTemplate {
  // ~ Static fields/initializers =====================================================================================
  private static final Log logger = LogFactory.getLog(SpringSecurityLdapTemplate.class);

  /** La constante NO_ATTRS. */
  public static final String[] NO_ATTRS = new String[0];

  // ~ Instance fields ================================================================================================

  /** Default search controls */
  private SearchControls searchControls = new SearchControls();

  // ~ Constructors ===================================================================================================

  /**
   * Construye un nuevo objeto de la clase
   * 
   * @param contextSource
   *           context source
   */
  public SpringSecurityLdapTemplateMock(ContextSource contextSource) {
    Assert.notNull(contextSource, "ContextSource cannot be null");
    setContextSource(contextSource);

    searchControls.setSearchScope(SearchControls.SUBTREE_SCOPE);
  }

  // ~ Methods ========================================================================================================

  /**
   * Performs an LDAP compare operation of the value of an attribute for a particular directory entry.
   * 
   * @param dn
   *           entry who's attribute is to be used
   * @param attributeName
   *           attribute who's value we want to compare
   * @param value
   *           value to be checked against the directory value
   * @return true if the supplied value matches that in the directory
   */
  public boolean compare(final String dn, final String attributeName, final Object value) {
    final String comparisonFilter = "(" + attributeName + "={0})";

    class LdapCompareCallback implements ContextExecutor {

      @SuppressWarnings("unchecked")
      public Object executeWithContext(DirContext ctx) throws NamingException {
        SearchControls ctls = new SearchControls();
        ctls.setReturningAttributes(NO_ATTRS);
        ctls.setSearchScope(SearchControls.OBJECT_SCOPE);

        NamingEnumeration results = ctx.search(dn, comparisonFilter, new Object[] { value }, ctls);

        return Boolean.valueOf(results.hasMore());
      }
    }

    Boolean matches = (Boolean) executeReadOnly(new LdapCompareCallback());

    return matches.booleanValue();
  }

  /**
   * Composes an object from the attributes of the given DN.
   * 
   * @param dn
   *           directory entry which will be read
   * @param attributesToRetrieve
   *           named attributes which will be retrieved from the directory entry.
   * @return object created by the mapper
   */
  public DirContextOperations retrieveEntry(final String dn, final String[] attributesToRetrieve) {

    return (DirContextOperations) executeReadOnly(new ContextExecutor() {
      public Object executeWithContext(DirContext ctx) throws NamingException {
        Attributes attrs = ctx.getAttributes(dn, attributesToRetrieve);

        // Object object = ctx.lookup(LdapUtils.getRelativeName(dn, ctx));

        return new DirContextAdapter(attrs, new DistinguishedName(dn), new DistinguishedName(ctx.getNameInNamespace()));
      }
    });
  }

  /**
   * Performs a search using the supplied filter and returns the union of the values of the named attribute found in all
   * entries matched by the search. Note that one directory entry may have several values for the attribute. Intended
   * for role searches and similar scenarios.
   * 
   * @param base
   *           DN to search in
   * @param filter
   *          search filter to use
   * @param params
   *           parameters to substitute in the search filter
   * @param attributeName
   *           attribute who's values are to be retrieved.
   * @return set of String values for the attribute as a union of the values found in all the matching entries.
   */
  @SuppressWarnings("unchecked")
  public Set searchForSingleAttributeValues(final String base, final String filter, final Object[] params,
      final String attributeName) {
    // Escape the params acording to RFC2254
    Object[] encodedParams = new String[params.length];

    for (int i = 0; i < params.length; i++) {
      encodedParams[i] = LdapEncoder.filterEncode(params[i].toString());
    }

    String formattedFilter = MessageFormat.format(filter, encodedParams);
    logger.debug("Using filter: " + formattedFilter);

    final HashSet set = new HashSet();

    ContextMapper roleMapper = new ContextMapper() {
      public Object mapFromContext(Object ctx) {
        DirContextAdapter adapter = (DirContextAdapter) ctx;
        String[] values = adapter.getStringAttributes(attributeName);
        if (values == null || values.length == 0) {
          logger.debug("No attribute value found for '" + attributeName + "'");
        } else {
          set.addAll(Arrays.asList(values));
        }
        return null;
      }
    };

    SearchControls ctls = new SearchControls();
    ctls.setSearchScope(searchControls.getSearchScope());
    ctls.setReturningAttributes(new String[] { attributeName });
    ctls.setReturningObjFlag(false);

    search(base, formattedFilter, ctls, roleMapper);

    return set;
  }

  /**
   * Performs a search, with the requirement that the search shall return a single directory entry, and uses the
   * supplied mapper to create the object from that entry.
   * <p>
   * Ignores <tt>PartialResultException</tt> if thrown, for compatibility with Active Directory (see
   * 
   * @param base
   *           search base, relative to the base context supplied by the context source.
   * @param filter
   *           LDAP search filter
   * @param params
   *          parameters to be substituted in the search.
   * @return a DirContextOperations instance created from the matching entry.
   *         {@link LdapTemplate#setIgnorePartialResultException(boolean)}).
   */
  public DirContextOperations searchForSingleEntry(final String base, final String filter, final Object[] params) {

    logger.info("Searching for single entry...");

    return (DirContextOperations) executeReadOnly(new ContextExecutor() {
      @SuppressWarnings("unchecked")
      public Object executeWithContext(DirContext ctx) throws NamingException {

        DistinguishedName ctxBaseDn = new DistinguishedName(ctx.getNameInNamespace());
        System.out.println("ctxBaseDN: " + ctxBaseDn.toString());
        NamingEnumeration resultsEnum = ctx.search(base, filter, params, searchControls);
        Set results = new HashSet();

        while (resultsEnum.hasMoreElements()) {
          try {
            SearchResult searchResult = (SearchResult) resultsEnum.next();
            StringBuffer dn = new StringBuffer(searchResult.getNameInNamespace());
            System.out.println("result: " + dn.toString());
            results.add(new DirContextAdapter(searchResult.getAttributes(), new DistinguishedName(dn.toString()),
                ctxBaseDn));
          } catch (PartialResultException e) {
            logger.info("Ignoring PartialResultException");
          }
        }

        if (results.size() == 0) {
          resultsEnum = ctx.search(base, filter, params, searchControls);
          results = new HashSet();

          while (resultsEnum.hasMoreElements()) {
            try {
              SearchResult searchResult = (SearchResult) resultsEnum.next();
              StringBuffer dn = new StringBuffer(searchResult.getName());
              System.out.println("result: " + dn.toString());
              if (base.length() > 0) {
                dn.append(",");
                dn.append(base);
              }
              results.add(new DirContextAdapter(searchResult.getAttributes(), new DistinguishedName(dn.toString()),
                  ctxBaseDn));
            } catch (PartialResultException e) {
              logger.info("Ignoring PartialResultException");
            }
          }
          if (results.size() == 0) throw new IncorrectResultSizeDataAccessException(1, 0);
        }
        return results.toArray()[0];
      }
    });
  }

  // public DirContextOperations searchForSingleEntry(final String base, final String filter, final Object[] params) {
  //
  // return (DirContextOperations) executeReadOnly(new ContextExecutor() {
  // public Object executeWithContext(DirContext ctx) throws NamingException {
  //
  // try {
  // NamingEnumeration results = ctx.search(base, filter, params, searchControls);
  //
  // if (!results.hasMoreElements()) throw new IncorrectResultSizeDataAccessException(1, 0);
  //
  // SearchResult searchResult = (SearchResult) results.next();
  // logger.debug("searchResult=" + searchResult);
  //
  // if (results.hasMoreElements()) {
  // if (!searchResult.equals((SearchResult) results.next()))
  // throw new IncorrectResultSizeDataAccessException(1, 2);
  // }
  //
  // // Work out the DN of the matched entry
  // StringBuffer dn = new StringBuffer(searchResult.getName());
  //
  // if (base.length() > 0) {
  // dn.append(",");
  // dn.append(base);
  // }
  //
  // return new DirContextAdapter(searchResult.getAttributes(), new DistinguishedName(dn.toString()),
  // new DistinguishedName(ctx.getNameInNamespace()));
  // } catch (Exception e) {
  // NamingException ne = new NamingException(e.getMessage());
  // ne.setStackTrace(e.getStackTrace());
  // throw ne;
  // }
  // }
  // });
  // }

  /**
   * Asigna search controls which will be used for search operations by the template.
   * 
   * @param searchControls
   *           SearchControls instance which will be cached in the template.
   */
  public void setSearchControls(SearchControls searchControls) {
    this.searchControls = searchControls;
  }
}
