package es.csa.fdt.cyldigital.tasks;

/*
 * #%L
 * Librerías del núcleo para Web - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.Calendar;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.NoResultException;
import javax.persistence.NonUniqueResultException;
import javax.persistence.Query;

import org.hibernate.HibernateException;

import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.quartz.SchedulerException;

import org.springframework.context.ApplicationContext;
import org.springframework.orm.jpa.EntityManagerHolder;
import org.springframework.scheduling.quartz.QuartzJobBean;
import org.springframework.transaction.support.TransactionSynchronizationManager;

import es.csa.fdt.cyldigital.core.service.ParameterServiceImpl;
import es.csa.fdt.cyldigital.notificacion.NotificationService;
import es.csa.fdt.cyldigital.servicios.entity.ActividadExterna;
import es.csa.fdt.cyldigital.servicios.entity.Curso;

/**
 * Clase CheckOpenenedActivitiesJob.
 */
public class CheckOpenenedActivitiesJob extends QuartzJobBean {
	private static final Logger logger = Logger.getLogger(CheckOpenenedActivitiesJob.class.getName());
	
	@Override
	protected final void executeInternal(JobExecutionContext ctx) throws JobExecutionException {
		ApplicationContext appContext = null;
	    EntityManagerFactory emf = null;
		EntityManager em = null;
		
		try {
			appContext = (ApplicationContext)ctx.getScheduler().getContext().get("applicationContext");

			emf = (EntityManagerFactory)appContext.getBean("entityManagerFactory", EntityManagerFactory.class);
			
			em = emf.createEntityManager();
			
			TransactionSynchronizationManager.bindResource(emf, new EntityManagerHolder(em));
			
			executeTransactional(appContext, em);
		}
		catch (HibernateException e) {
			logger.log(Level.SEVERE, "CheckOpenenedActivitiesJob:executeInternal -> " +
									 "Excepción de hibernate al ejecutar una tarea: " +
									 e.getLocalizedMessage());
			
			e.printStackTrace();
		}
		catch (SchedulerException e) {
			logger.log(Level.SEVERE, "CheckOpenenedActivitiesJob:executeInternal -> " +
									 "Error en el planificador de tareas: " +
									 e.getLocalizedMessage());

			e.printStackTrace();
		}
		catch (RuntimeException e) {
			logger.log(Level.SEVERE, "CheckOpenenedActivitiesJob:executeInternal -> " +
									 "Error en el general al ejecutar una tarea: " +
									 e.getLocalizedMessage());

			e.printStackTrace();
		}
		finally {
			 if (em != null)
				 em.close();

			 TransactionSynchronizationManager.unbindResource(emf);
		}
	}
	
	@SuppressWarnings({ "unchecked", "rawtypes" })
	private void executeTransactional(ApplicationContext appContext, EntityManager em) throws JobExecutionException {
		ParameterServiceImpl parameterService = new ParameterServiceImpl();
		parameterService.setEntityManager(em);
		
		Integer daysToAdvise = null;
		Integer daysToExecution = null;
		
		Boolean mailActivate = false;
		Boolean mailAutoCloseTask = false;
		
		try {
			//Extract settings for auto-close activities
			daysToAdvise = Integer.parseInt(parameterService.findOneWhere("nombre", "activities_autoclosetask_advertisement_days").getValor());
			daysToExecution = Integer.parseInt(parameterService.findOneWhere("nombre", "activities_autoclosetask_execution_days").getValor());
			
			//Extract setting for mailing
			mailActivate      = parameterService.findOneWhere("nombre", "mail_activate").getValor().contentEquals("1");
			mailAutoCloseTask = parameterService.findOneWhere("nombre", "mail_autoclosetask_activate").getValor().contentEquals("1");
		}
		catch (NoResultException e) {
			logger.log(Level.SEVERE, "CheckOpenenedActivitiesJob:executeTransactional -> " +
									 "Excepción al extraer parámetros de configuración (sin resultados): " +
									 e.getLocalizedMessage());
		}
		catch (NonUniqueResultException e) {
			logger.log(Level.SEVERE, "CheckOpenenedActivitiesJob:executeTransactional -> " +
									 "Excepción al extraer parámetros de configuración (no hoy un único resultado): " +
									 e.getLocalizedMessage());
		}
		catch (IllegalStateException e) {
			logger.log(Level.SEVERE, "CheckOpenenedActivitiesJob:executeTransactional -> " +
									 "Excepción al extraer parámetros de configuración: " +
									 e.getLocalizedMessage());
		}
		catch (NumberFormatException e) {
			logger.log(Level.SEVERE, "CheckOpenenedActivitiesJob:executeTransactional -> " +
									 "Excepción al convertir el valor de la cadena de un parámetro en número: " +
									 e.getLocalizedMessage());
		}
		catch (Exception e) {
			logger.log(Level.SEVERE, "CheckOpenenedActivitiesJob:executeTransactional -> " +
									 "Excepción general: " +
									 e.getLocalizedMessage());
		}
		
		if ((daysToAdvise != null) && (daysToExecution != null)) {
			Calendar dateToAdvise = Calendar.getInstance();
			Calendar dateToExecute = Calendar.getInstance();
			
			//Set day to advise
			dateToAdvise.add(Calendar.DATE, -daysToAdvise);
			//Set time at 23:59:59
			dateToAdvise.set(dateToAdvise.get(Calendar.YEAR), dateToAdvise.get(Calendar.MONTH), dateToAdvise.get(Calendar.DAY_OF_MONTH),
							 23, 59, 59);
			
			//Set day to execute
			dateToExecute.add(Calendar.DATE, -(daysToAdvise + daysToExecution));
			//Set time at 23:59:59
			dateToExecute.set(dateToExecute.get(Calendar.YEAR), dateToExecute.get(Calendar.MONTH), dateToExecute.get(Calendar.DAY_OF_MONTH),
							  23, 59, 59);
			
			//Create calendar to store end date of activities
			Calendar dateEndActivity = Calendar.getInstance();
			
			//Create variables to query to data base and notification service
			NotificationService ns = new NotificationService();
			String qrystr;
			Query query;
			
			//Process activities type Curso
			qrystr = "SELECT c " +
					 "FROM " + Curso.class.getName() + " c " +
					 "WHERE (c.fechaFin <= :dateToAdvise)) AND " +
					 	   "(c.cerrada = FALSE)";
			
		    query = em.createQuery(qrystr);
		    query.setParameter("dateToAdvise", dateToAdvise.getTime());
	
		    List<Curso> lc = query.getResultList();
		    
		    for (Curso c : lc) {
		    	dateEndActivity.setTime(c.getFechaFin());
		    	dateEndActivity.set(dateEndActivity.get(Calendar.YEAR),
		    						dateEndActivity.get(Calendar.MONTH),
		    						dateEndActivity.get(Calendar.DAY_OF_MONTH),
		    						23, 59, 59);

		    	if (mailActivate && mailAutoCloseTask && (dateEndActivity.compareTo(dateToAdvise) == 0))
		    		ns.notificarCursoAutoCerradoProximamenteByJob(appContext, parameterService, c, daysToAdvise, daysToExecution);
		    	else if (dateEndActivity.compareTo(dateToExecute) <= 0) {
		    		c.setCerrada(true);
		    		
		    		em.getTransaction().begin();
		    		em.persist(c);
		    		em.getTransaction().commit();
		    		
		    		if (mailActivate && mailAutoCloseTask)
		    			ns.notificarCursoAutoCerradoByJob(appContext, parameterService, c, (daysToAdvise + daysToExecution));
		    	}
		    }

		    //Process activities type ActividadExterna
		    qrystr = "SELECT c " +
		    		 "FROM " + ActividadExterna.class.getName() + " c " +
		    		 "WHERE (c.fechaFin <= :dateToAdvise)) AND " +
		    		 	   "(c.cerrada = FALSE)";

		    query = em.createQuery(qrystr);
		    query.setParameter("dateToAdvise", dateToAdvise.getTime());

		    List<ActividadExterna> le = query.getResultList();

		    for (ActividadExterna e : le) {
		    	dateEndActivity.setTime(e.getFechaFin());
		    	dateEndActivity.set(dateEndActivity.get(Calendar.YEAR),
		    						dateEndActivity.get(Calendar.MONTH),
		    						dateEndActivity.get(Calendar.DAY_OF_MONTH),
		    						23, 59, 59);
		    	
		    	if (mailActivate && mailAutoCloseTask && (dateEndActivity.compareTo(dateToAdvise) == 0))
		    		ns.notificarExternaAutoCerradaProximamenteByJob(appContext, parameterService, e, daysToAdvise, daysToExecution);
		    	else if (dateEndActivity.compareTo(dateToExecute) <= 0) {
		    		e.setCerrada(true);
		    		
		    		em.getTransaction().begin();
		    		em.persist(e);
		    		em.getTransaction().commit();
		    		
		    		if (mailActivate && mailAutoCloseTask)
		    			ns.notificarExternaAutoCerradaByJob(appContext, parameterService, e, (daysToAdvise + daysToExecution));
		    	}
		    }
		}
	}
}
