package es.csa.fdt.cyldigital.tasks;

/*
 * #%L
 * Librerías del núcleo para Web - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.Date;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Query;

import org.hibernate.HibernateException;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.quartz.SchedulerException;
import org.springframework.context.ApplicationContext;
import org.springframework.orm.jpa.EntityManagerHolder;
import org.springframework.scheduling.quartz.QuartzJobBean;
import org.springframework.transaction.support.TransactionSynchronizationManager;

import es.csa.fdt.cyldigital.notificacion.NotificationService;
import es.csa.fdt.cyldigital.tareas.entity.InformeTarea;
import es.csa.fdt.cyldigital.tareas.entity.InformeTarea.EstadoTarea;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado.Puesto;
import es.csa.fdt.cyldigital.usuarios.service.UsuarioService;

/**
 * Clase TransactionalQuartzTask.
 */
public abstract class TransactionalQuartzTask extends QuartzJobBean {

  private static final Logger logger = Logger.getLogger(TransactionalQuartzTask.class.getName());

  /**
   * Metodo principal.
   * 
   * @param ctx
   *           ctx
   * @throws JobExecutionException
   *           the job execution exception
   */
  @Override
  protected final void executeInternal(JobExecutionContext ctx) throws JobExecutionException {
    ApplicationContext appContext = null;
    EntityManagerFactory emf = null;
    EntityManager em = null;
    InformeTarea informe = new InformeTarea();

    try {
      preExecution(informe);
      appContext = (ApplicationContext) ctx.getScheduler().getContext().get("applicationContext");
      emf = (EntityManagerFactory) appContext.getBean("entityManagerFactory", EntityManagerFactory.class);
      em = emf.createEntityManager();
      TransactionSynchronizationManager.bindResource(emf, new EntityManagerHolder(em));

      executeTransactional(ctx, em, informe);

    } catch (HibernateException e) {
      informe.setEstadoTarea(EstadoTarea.ERROR);
      informe.setMensaje(e.getLocalizedMessage());
      logger.log(Level.SEVERE, "Excepción de hibernate al ejecutar una tarea: " + e.getLocalizedMessage());
      e.printStackTrace();
    } catch (SchedulerException e) {
      informe.setEstadoTarea(EstadoTarea.ERROR);
      informe.setMensaje(e.getLocalizedMessage());
      logger.log(Level.SEVERE, "Error en el planificador de tareas: " + e.getLocalizedMessage());
      e.printStackTrace();
    } catch (RuntimeException e) {
      informe.setEstadoTarea(EstadoTarea.ERROR);
      informe.setMensaje(e.getLocalizedMessage());
      logger.log(Level.SEVERE, "Error en el general al ejecutar una tarea: " + e.getLocalizedMessage());
      e.printStackTrace();
    } finally {
      postExecution(ctx, em, informe);
      if (em != null) em.close();
      TransactionSynchronizationManager.unbindResource(emf);
    }

  }

  /**
   * Metodo a implementar por las tareas concretas.
   * 
   * @param ctx
   *          Contexto del trabajo
   * @param em
   *          Entity Manager
   * @param informe
   *          Informe de ejecución de la tarea
   * @throws JobExecutionException
   *           Excepción de la tarea
   */
  protected abstract void executeTransactional(JobExecutionContext ctx, EntityManager em, InformeTarea informe)
      throws JobExecutionException;

  /**
   * Pre execution.
   * 
   * @param informe
   *           informe
   */
  protected void preExecution(InformeTarea informe) {
    informe.setFechaInicio(new Date());
    informe.setEstadoTarea(EstadoTarea.NOT_DONE);
    informe.setNombre(getTaskName());
  }

  /**
   * Post execution.
   * 
   * @param ctx
   *           ctx
   * @param em
   *           em
   * @param informe
   *           informe
   */
  protected void postExecution(JobExecutionContext ctx, EntityManager em, InformeTarea informe) {
    informe.setFechaFin(new Date());
    if (informe.getEstadoTarea().equals(EstadoTarea.NOT_DONE)) informe.setEstadoTarea(EstadoTarea.OK);
    try {
      em.getTransaction().begin();
      em.persist(informe);
      em.getTransaction().commit();
    } catch (RuntimeException e) {
      logger.log(Level.SEVERE, "Error al registrar la información de una tarea en la base de datos");
      e.printStackTrace();
    }

    if (informe.getEstadoTarea().equals(EstadoTarea.ERROR)) {
      notificarResultadoEmailAdministradores(em, informe);
    }
  }

  private void notificarResultadoEmailAdministradores(EntityManager em, InformeTarea informe) {
    String consulta = "SELECT e FROM " + Empleado.class.getName() + " e WHERE puesto = :valor1";
    Query query = em.createQuery(consulta);
    query.setParameter("valor1", Puesto.ADMIN);
    List<Usuario> administradores = query.getResultList();
    if (!administradores.isEmpty()) {
      NotificationService<UsuarioService> n = new NotificationService<UsuarioService>();
      n.notificarResultadoTareaProgramada(em, administradores, informe);
    }
  }

  /**
   * Devuelve task name.
   * 
   * @return the task name
   */
  protected String getTaskName() {
    return this.getClass().getCanonicalName();
  }

}
